@@ -4,6 +4,7 @@
   connected adaptive quadtrees or octrees in parallel.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
   Written by Carsten Burstedde, Lucas C. Wilcox, and Tobin Isaac
 
   p4est is free software; you can redistribute it and/or modify
@@ -260,6 +261,7 @@ main (int argc, char **argv)
   int                 first_argc;
   int                 test_multiple_orders;
   int                 skip_nodes, skip_lnodes;
+  int                 repartition_lnodes;
 
   /* initialize MPI and p4est internals */
   mpiret = sc_MPI_Init (&argc, &argv);
@@ -316,6 +318,9 @@ main (int argc, char **argv)
                          "Also time lnodes for orders 2, 4, and 8");
   sc_options_add_switch (opt, 0, "skip-nodes", &skip_nodes, "Skip nodes");
   sc_options_add_switch (opt, 0, "skip-lnodes", &skip_lnodes, "Skip lnodes");
+  sc_options_add_switch (opt, 0, "repartition-lnodes",
+                         &repartition_lnodes,
+                         "Repartition to load-balance lnodes");
 
   first_argc = sc_options_parse (p4est_package_id, SC_LP_DEFAULT,
                                  opt, argc, argv);
@@ -448,46 +453,46 @@ main (int argc, char **argv)
 
     /* print all available regression tests */
     if (generate) {
-      const char         *config_name = NULL;
+      const char         *config_gname = NULL;
 
       P4EST_GLOBAL_PRODUCTION ("Checksum regression tests available:\n");
       for (r = regression; r->config != P4EST_CONFIG_NULL; ++r) {
         switch (r->config) {
         case P4EST_CONFIG_PERIODIC:
-          config_name = "periodic";
+          config_gname = "periodic";
           break;
 #ifndef P4_TO_P8
         case P4EST_CONFIG_THREE:
-          config_name = "three";
+          config_gname = "three";
           break;
         case P4EST_CONFIG_MOEBIUS:
-          config_name = "moebius";
+          config_gname = "moebius";
           break;
         case P4EST_CONFIG_STAR:
-          config_name = "star";
+          config_gname = "star";
           break;
 #else
         case P4EST_CONFIG_ROTWRAP:
-          config_name = "rotwrap";
+          config_gname = "rotwrap";
           break;
         case P4EST_CONFIG_TWOCUBES:
-          config_name = "twocubes";
+          config_gname = "twocubes";
           break;
         case P4EST_CONFIG_ROTCUBES:
-          config_name = "rotcubes";
+          config_gname = "rotcubes";
           break;
         case P4EST_CONFIG_SHELL:
-          config_name = "shell";
+          config_gname = "shell";
           break;
 #endif
         default:
-          config_name = "unit";
+          config_gname = "unit";
           break;
         }
         P4EST_GLOBAL_PRODUCTIONF ("mpirun -np %3d %s%s -c %10s -l %2d\n",
-                                  r->mpisize, opt->program_path,
+                                  r->mpisize, P4EST_STRING "_timings",
                                   oldschool ? " --oldschool" : "",
-                                  config_name, r->level);
+                                  config_gname, r->level);
       }
     }
   }
@@ -636,6 +641,12 @@ main (int argc, char **argv)
     p4est_nodes_destroy (nodes);
   }
 
+  if (!skip_lnodes && repartition_lnodes) {
+    p4est_partition_lnodes (p4est, ghost, 1, 0);
+    p4est_ghost_destroy (ghost);
+    ghost = p4est_ghost_new (p4est, P4EST_CONNECT_FULL);
+  }
+
   /* time the lnode numbering */
   if (!skip_lnodes) {
     sc_flops_snap (&fi, &snapshot);
@@ -649,12 +660,22 @@ main (int argc, char **argv)
   }
 
   if (test_multiple_orders) {
+    if (repartition_lnodes) {
+      p4est_partition_lnodes (p4est, ghost, 3, 0);
+      p4est_ghost_destroy (ghost);
+      ghost = p4est_ghost_new (p4est, P4EST_CONNECT_FULL);
+    }
     sc_flops_snap (&fi, &snapshot);
     lnodes = p4est_lnodes_new (p4est, ghost, 3);
     sc_flops_shot (&fi, &snapshot);
     sc_stats_set1 (&stats[TIMINGS_LNODES3], snapshot.iwtime, "L-Nodes 3");
     p4est_lnodes_destroy (lnodes);
 
+    if (repartition_lnodes) {
+      p4est_partition_lnodes (p4est, ghost, 7, 0);
+      p4est_ghost_destroy (ghost);
+      ghost = p4est_ghost_new (p4est, P4EST_CONNECT_FULL);
+    }
     sc_flops_snap (&fi, &snapshot);
     lnodes = p4est_lnodes_new (p4est, ghost, 7);
     sc_flops_shot (&fi, &snapshot);
@@ -713,7 +734,7 @@ main (int argc, char **argv)
 
   /* calculate and print timings */
   sc_stats_compute (mpi->mpicomm, TIMINGS_NUM_STATS, stats);
-  sc_stats_print (p4est_package_id, SC_LP_STATISTICS,
+  sc_stats_print (p4est_package_id, SC_LP_ESSENTIAL,
                   TIMINGS_NUM_STATS, stats, 1, 1);
 
   /* destroy the p4est and its connectivity structure */
