@@ -3,6 +3,7 @@
   The SC Library provides support for parallel scientific applications.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
 
   The SC Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -20,7 +21,7 @@
   02110-1301, USA.
 */
 
-#include <sc.h>
+#include <sc_private.h>
 
 #ifdef SC_HAVE_SIGNAL_H
 #include <signal.h>
@@ -38,6 +39,8 @@ typedef void        (*sc_sig_t) (int);
 #endif
 #endif
 
+#include <errno.h>
+
 #ifdef SC_ENABLE_PTHREAD
 #include <pthread.h>
 #endif
@@ -50,6 +53,8 @@ typedef struct sc_package
   int                 log_indent;
   int                 malloc_count;
   int                 free_count;
+  int                 rc_active;
+  int                 abort_mismatch;
   const char         *name;
   const char         *full;
 #ifdef SC_ENABLE_PTHREAD
@@ -91,6 +96,8 @@ int                 sc_trace_prio = SC_LP_STATISTICS;
 
 static int          default_malloc_count = 0;
 static int          default_free_count = 0;
+static int          default_rc_active = 0;
+static int          default_abort_mismatch = 1;
 
 static int          sc_identifier = -1;
 static sc_MPI_Comm  sc_mpicomm = sc_MPI_COMM_NULL;
@@ -99,6 +106,9 @@ static FILE        *sc_log_stream = NULL;
 static sc_log_handler_t sc_default_log_handler = sc_log_handler;
 static int          sc_default_log_threshold = SC_LP_THRESHOLD;
 
+static void         sc_abort_handler (void);
+static sc_abort_handler_t sc_default_abort_handler = sc_abort_handler;
+
 static int          sc_signals_caught = 0;
 static sc_sig_t     system_int_handler = NULL;
 static sc_sig_t     system_segv_handler = NULL;
@@ -142,27 +152,57 @@ sc_package_mutex (int package)
   }
 }
 
-static inline void
+#endif /* SC_ENABLE_PTHREAD */
+
+void
 sc_package_lock (int package)
 {
+#ifdef SC_ENABLE_PTHREAD
   pthread_mutex_t    *mutex = sc_package_mutex (package);
   int                 pth;
 
   pth = pthread_mutex_lock (mutex);
   sc_check_abort_thread (pth == 0, package, "sc_package_lock");
+#endif
 }
 
-static inline void
+void
 sc_package_unlock (int package)
 {
+#ifdef SC_ENABLE_PTHREAD
   pthread_mutex_t    *mutex = sc_package_mutex (package);
   int                 pth;
 
   pth = pthread_mutex_unlock (mutex);
   sc_check_abort_thread (pth == 0, package, "sc_package_unlock");
+#endif
 }
 
-#endif /* SC_ENABLE_PTHREAD */
+void
+sc_package_rc_count_add (int package_id, int toadd)
+{
+  int                *pcount;
+#ifdef SC_ENABLE_DEBUG
+  int                 newvalue;
+#endif
+
+  if (package_id == -1) {
+    pcount = &default_rc_active;
+  }
+  else {
+    SC_ASSERT (sc_package_is_registered (package_id));
+    pcount = &sc_packages[package_id].rc_active;
+  }
+
+  sc_package_lock (package_id);
+#ifdef SC_ENABLE_DEBUG
+  newvalue =
+#endif
+    *pcount += toadd;
+  sc_package_unlock (package_id);
+
+  SC_ASSERT (newvalue >= 0);
+}
 
 static void
 sc_signal_handler (int sig)
@@ -275,17 +315,222 @@ sc_free_count (int package)
   return &sc_packages[package].free_count;
 }
 
+#ifdef SC_ENABLE_MEMALIGN
+
+/* *INDENT-OFF* */
+static void        *
+sc_malloc_aligned (size_t alignment, size_t size)
+SC_ATTR_ALIGN (SC_MEMALIGN_BYTES);
+/* *INDENT-ON* */
+
+static void        *
+sc_malloc_aligned (size_t alignment, size_t size)
+{
+  /* minimum requirements on alignment */
+  SC_ASSERT (sizeof (char **) == sizeof (void *));
+  SC_ASSERT (sizeof (char **) >= sizeof (size_t));
+  SC_ASSERT (alignment > 0 && alignment % sizeof (void *) == 0);
+  SC_ASSERT (alignment == SC_MEMALIGN_BYTES);
+
+#if defined SC_HAVE_ANY_MEMALIGN && defined SC_HAVE_POSIX_MEMALIGN
+  {
+    void               *data = NULL;
+    int                 err = posix_memalign (&data, alignment, size);
+    SC_CHECK_ABORTF (err != ENOMEM, "Insufficient memory (malloc size %llu)",
+                     (long long unsigned) size);
+    SC_CHECK_ABORTF (err != EINVAL, "Alignment %llu is not a power of two"
+                     "or not a multiple of sizeof (void *)",
+                     (long long unsigned) alignment);
+    SC_CHECK_ABORTF (err == 0, "Return of %d from posix_memalign", err);
+    return data;
+  }
+#elif defined SC_HAVE_ANY_MEMALIGN && defined SC_HAVE_ALIGNED_ALLOC
+  {
+    void               *data = aligned_alloc (alignment, size);
+    SC_CHECK_ABORT (data != NULL || size == 0,
+                    "Returned NULL from aligned_alloc");
+    return data;
+  }
+#else
+  {
+#if 0
+    /* adapted from PetscMallocAlign */
+    int                *datastart = malloc (size + 2 * alignment);
+    int                 shift = ((uintptr_t) datastart) % alignment;
+
+    shift = (2 * alignment - shift) / sizeof (int);
+    datastart[shift - 1] = shift;
+    datastart += shift;
+    return (void *) datastart;
+#endif
+    /* We pad to achieve alignment, then write the original pointer and data
+     * size up front, then the real data shifted by at most alignment - 1
+     * bytes.  This way there is always at least one stop byte at the end that
+     * we can use for debugging. */
+    const ptrdiff_t     extrasize = (const ptrdiff_t) (2 * sizeof (char **));
+    const ptrdiff_t     signalign = (const ptrdiff_t) alignment;
+    const size_t        alloc_size = extrasize + size + alignment;
+    char               *alloc_ptr = (char *) malloc (alloc_size);
+    char               *ptr;
+    ptrdiff_t           shift, modu;
+
+    SC_CHECK_ABORT (alloc_ptr != NULL, "Returned NULL from malloc");
+
+    /* compute shift to the right where we put the actual data */
+    modu = ((ptrdiff_t) alloc_ptr + extrasize) % signalign;
+    shift = (signalign - modu) % signalign;
+    SC_ASSERT (0 <= shift && shift < signalign);
+
+    /* make sure the resulting pointer is fine */
+    ptr = alloc_ptr + (extrasize + shift);
+    SC_ASSERT ((ptrdiff_t) ptr % signalign == 0);
+
+    /* memorize the original pointer that we got from malloc and fill up */
+    SC_ARG_ALIGN (ptr, char *, SC_MEMALIGN_BYTES);
+
+    /* remember parameters of allocation for later use */
+    ((char **) ptr)[-1] = alloc_ptr;
+    ((char **) ptr)[-2] = (char *) size;
+#ifdef SC_ENABLE_DEBUG
+    memset (alloc_ptr, (char) -2, shift);
+    SC_ASSERT (ptr + ((ptrdiff_t) size + signalign - shift) ==
+               alloc_ptr + alloc_size);
+    memset (ptr + size, (char) -2, signalign - shift);
+#endif
+
+    /* and we are done */
+    return (void *) ptr;
+  }
+#endif
+}
+
+static void
+sc_free_aligned (void *ptr, size_t alignment)
+{
+  /* minimum requirements on alignment */
+  SC_ASSERT (sizeof (char **) == sizeof (void *));
+  SC_ASSERT (sizeof (char **) >= sizeof (size_t));
+  SC_ASSERT (alignment > 0 && alignment % sizeof (void *) == 0);
+
+#if defined SC_HAVE_ANY_MEMALIGN && \
+   (defined SC_HAVE_POSIX_MEMALIGN || defined SC_HAVE_ALIGNED_ALLOC)
+  free (ptr);
+#else
+  {
+#if 0
+    int                *datastart = ptr;
+    int                 shift = datastart[-1];
+
+    datastart -= shift;
+    free ((void *) datastart);
+#endif
+    /* this mirrors the function sc_malloc_aligned above */
+    char               *alloc_ptr;
+#ifdef SC_ENABLE_DEBUG
+    const ptrdiff_t     extrasize = (const ptrdiff_t) (2 * sizeof (char **));
+    const ptrdiff_t     signalign = (const ptrdiff_t) alignment;
+    ptrdiff_t           shift, modu, ssize, i;
+#endif
+
+    /* we excluded these cases earlier */
+    SC_ASSERT (ptr != NULL);
+    SC_ASSERT ((ptrdiff_t) ptr % signalign == 0);
+
+    alloc_ptr = ((char **) ptr)[-1];
+    SC_ASSERT (alloc_ptr != NULL);
+
+#ifdef SC_ENABLE_DEBUG
+    /* compute shift to the right where we put the actual data */
+    ssize = (ptrdiff_t) ((char **) ptr)[-2];
+    modu = ((ptrdiff_t) alloc_ptr + extrasize) % signalign;
+    shift = (signalign - modu) % signalign;
+    SC_ASSERT (0 <= shift && shift < signalign);
+    SC_ASSERT ((char *) ptr == alloc_ptr + (extrasize + shift));
+    for (i = 0; i < shift; ++i) {
+      SC_ASSERT (alloc_ptr[i] == (char) -2);
+    }
+    for (i = 0; i < signalign - shift; ++i) {
+      SC_ASSERT (((char *) ptr)[ssize + i] == (char) -2);
+    }
+#endif
+
+    /* free the original pointer */
+    free (alloc_ptr);
+  }
+#endif
+}
+
+/* *INDENT-OFF* */
+static void        *
+sc_realloc_aligned (void *ptr, size_t alignment, size_t size)
+SC_ATTR_ALIGN (SC_MEMALIGN_BYTES);
+/* *INDENT-ON* */
+
+static void        *
+sc_realloc_aligned (void *ptr, size_t alignment, size_t size)
+{
+  /* minimum requirements on alignment */
+  SC_ASSERT (sizeof (char **) == sizeof (void *));
+  SC_ASSERT (sizeof (char **) >= sizeof (size_t));
+  SC_ASSERT (alignment > 0 && alignment % sizeof (void *) == 0);
+  SC_ASSERT (alignment == SC_MEMALIGN_BYTES);
+
+#if defined SC_HAVE_ANY_MEMALIGN && \
+   (defined SC_HAVE_POSIX_MEMALIGN || defined SC_HAVE_ALIGNED_ALLOC)
+  /* the result is no longer aligned */
+  return realloc (ptr, size);
+#else
+  {
+#ifdef SC_ENABLE_DEBUG
+    const ptrdiff_t     signalign = (const ptrdiff_t) alignment;
+#endif
+    size_t              old_size, min_size;
+    void               *new_ptr;
+
+    /* we excluded these cases earlier */
+    SC_ASSERT (ptr != NULL && size > 0);
+    SC_ASSERT ((ptrdiff_t) ptr % signalign == 0);
+
+    /* back out the previously allocated size */
+    old_size = (size_t) ((char **) ptr)[-2];
+
+    /* create new memory while the old memory is still around */
+    new_ptr = sc_malloc_aligned (alignment, size);
+
+    /* copy data */
+    min_size = SC_MIN (old_size, size);
+    memcpy (new_ptr, ptr, min_size);
+#ifdef SC_ENABLE_DEBUG
+    memset ((char *) new_ptr + min_size, (char) -3, size - min_size);
+#endif
+
+    /* free old memory and return new pointer */
+    sc_free_aligned (ptr, alignment);
+    return new_ptr;
+  }
+#endif
+}
+
+#endif /* SC_ENABLE_MEMALIGN */
+
 void               *
 sc_malloc (int package, size_t size)
 {
   void               *ret;
   int                *malloc_count = sc_malloc_count (package);
 
+  /* allocate memory */
+#if defined SC_ENABLE_MEMALIGN
+  ret = sc_malloc_aligned (SC_MEMALIGN_BYTES, size);
+#else
   ret = malloc (size);
   if (size > 0) {
-    SC_CHECK_ABORT (ret != NULL, "Allocation");
+    SC_CHECK_ABORTF (ret != NULL, "Allocation (malloc size %lli)",
+                     (long long int) size);
   }
+#endif
 
+  /* count the allocations */
 #ifdef SC_ENABLE_PTHREAD
   sc_package_lock (package);
 #endif
@@ -308,11 +553,19 @@ sc_calloc (int package, size_t nmemb, size_t size)
   void               *ret;
   int                *malloc_count = sc_malloc_count (package);
 
+  /* allocate memory */
+#if defined SC_ENABLE_MEMALIGN
+  ret = sc_malloc_aligned (SC_MEMALIGN_BYTES, nmemb * size);
+  memset (ret, 0, nmemb * size);
+#else
   ret = calloc (nmemb, size);
   if (nmemb * size > 0) {
-    SC_CHECK_ABORT (ret != NULL, "Allocation");
+    SC_CHECK_ABORTF (ret != NULL, "Allocation (calloc size %lli)",
+                     (long long int) size);
   }
+#endif
 
+  /* count the allocations */
 #ifdef SC_ENABLE_PTHREAD
   sc_package_lock (package);
 #endif
@@ -342,8 +595,13 @@ sc_realloc (int package, void *ptr, size_t size)
   else {
     void               *ret;
 
+#if defined SC_ENABLE_MEMALIGN
+    ret = sc_realloc_aligned (ptr, SC_MEMALIGN_BYTES, size);
+#else
     ret = realloc (ptr, size);
-    SC_CHECK_ABORT (ret != NULL, "Reallocation");
+    SC_CHECK_ABORTF (ret != NULL, "Reallocation (realloc size %lli)",
+                     (long long int) size);
+#endif
 
     return ret;
   }
@@ -369,7 +627,11 @@ sc_strdup (int package, const char *s)
 void
 sc_free (int package, void *ptr)
 {
-  if (ptr != NULL) {
+  if (ptr == NULL) {
+    return;
+  }
+  else {
+    /* uncount the allocations */
     int                *free_count = sc_free_count (package);
 
 #ifdef SC_ENABLE_PTHREAD
@@ -380,7 +642,13 @@ sc_free (int package, void *ptr)
     sc_package_unlock (package);
 #endif
   }
+
+  /* free memory */
+#if defined SC_ENABLE_MEMALIGN
+  sc_free_aligned (ptr, SC_MEMALIGN_BYTES);
+#else
   free (ptr);
+#endif
 }
 
 int
@@ -398,19 +666,47 @@ sc_memory_status (int package)
   }
 }
 
+void
+sc_package_set_abort_alloc_mismatch (int package_id, int set_abort)
+{
+  if (package_id == -1) {
+    default_abort_mismatch = set_abort;
+  }
+  else {
+    sc_package_t       *p;
+
+    SC_ASSERT (sc_package_is_registered (package_id));
+    p = sc_packages + package_id;
+    p->abort_mismatch = set_abort;
+  }
+}
+
 void
 sc_memory_check (int package)
 {
   sc_package_t       *p;
 
-  if (package == -1)
-    SC_CHECK_ABORT (default_malloc_count == default_free_count,
-                    "Memory balance (default)");
+  if (package == -1) {
+    SC_CHECK_ABORT (default_rc_active == 0, "Leftover references (default)");
+    if (default_abort_mismatch) {
+      SC_CHECK_ABORT (default_malloc_count == default_free_count,
+                      "Memory balance (default)");
+    }
+    else if (default_malloc_count != default_free_count) {
+      SC_GLOBAL_LERROR ("Memory balance (default)\n");
+    }
+  }
   else {
     SC_ASSERT (sc_package_is_registered (package));
     p = sc_packages + package;
-    SC_CHECK_ABORTF (p->malloc_count == p->free_count,
-                     "Memory balance (%s)", p->name);
+    SC_CHECK_ABORTF (p->rc_active == 0, "Leftover references (%s)", p->name);
+    if (p->abort_mismatch) {
+      SC_CHECK_ABORTF (p->malloc_count == p->free_count,
+                       "Memory balance (%s)", p->name);
+    }
+    else if (p->malloc_count != p->free_count) {
+      SC_GLOBAL_LERRORF ("Memory balance (%s)\n", p->name);
+    }
   }
 }
 
@@ -478,8 +774,8 @@ sc_set_log_defaults (FILE * log_stream,
     sc_default_log_threshold = SC_LP_THRESHOLD;
   }
   else {
-    SC_ASSERT (log_threshold >= SC_LP_ALWAYS
-               && log_threshold <= SC_LP_SILENT);
+    SC_ASSERT (log_threshold >= SC_LP_ALWAYS &&
+               log_threshold <= SC_LP_SILENT);
     sc_default_log_threshold = log_threshold;
   }
 
@@ -600,8 +896,22 @@ sc_log_indent_pop_count (int package, int count)
 #endif
 }
 
+void
+sc_set_abort_handler (sc_abort_handler_t abort_handler)
+{
+  sc_default_abort_handler = abort_handler != NULL ? abort_handler :
+    sc_abort_handler;
+}
+
 void
 sc_abort (void)
+{
+  sc_default_abort_handler ();
+  abort ();                     /* if the user supplied callback incorrecty returns, abort */
+}
+
+static void
+sc_abort_handler (void)
 {
   if (0) {
   }
@@ -705,8 +1015,8 @@ sc_package_register (sc_log_handler_t log_handler, int log_threshold,
   int                 new_package_id = -1;
 
   SC_CHECK_ABORT (log_threshold == SC_LP_DEFAULT ||
-                  (log_threshold >= SC_LP_ALWAYS
-                   && log_threshold <= SC_LP_SILENT),
+                  (log_threshold >= SC_LP_ALWAYS &&
+                   log_threshold <= SC_LP_SILENT),
                   "Invalid package log threshold");
   SC_CHECK_ABORT (strcmp (name, "default"), "Package default forbidden");
   SC_CHECK_ABORT (strchr (name, ' ') == NULL,
@@ -748,6 +1058,7 @@ sc_package_register (sc_log_handler_t log_handler, int log_threshold,
       p->log_indent = 0;
       p->malloc_count = 0;
       p->free_count = 0;
+      p->rc_active = 0;
       p->name = NULL;
       p->full = NULL;
     }
@@ -759,6 +1070,8 @@ sc_package_register (sc_log_handler_t log_handler, int log_threshold,
   new_package->log_indent = 0;
   new_package->malloc_count = 0;
   new_package->free_count = 0;
+  new_package->rc_active = 0;
+  new_package->abort_mismatch = 1;
   new_package->name = name;
   new_package->full = full;
 #ifdef SC_ENABLE_PTHREAD
@@ -782,6 +1095,22 @@ sc_package_is_registered (int package_id)
           sc_packages[package_id].is_registered);
 }
 
+void
+sc_package_set_verbosity (int package_id, int log_priority)
+{
+  sc_package_t       *p;
+
+  SC_CHECK_ABORT (sc_package_is_registered (package_id),
+                  "Package id is not registered");
+  SC_CHECK_ABORT (log_priority == SC_LP_DEFAULT ||
+                  (log_priority >= SC_LP_ALWAYS &&
+                   log_priority <= SC_LP_SILENT),
+                  "Invalid package log threshold");
+
+  p = sc_packages + package_id;
+  p->log_threshold = log_priority;
+}
+
 void
 sc_package_unregister (int package_id)
 {
@@ -799,6 +1128,7 @@ sc_package_unregister (int package_id)
   p->log_handler = NULL;
   p->log_threshold = SC_LP_DEFAULT;
   p->malloc_count = p->free_count = 0;
+  p->rc_active = 0;
 #ifdef SC_ENABLE_PTHREAD
   i = pthread_mutex_destroy (&p->mutex);
   SC_CHECK_ABORTF (i == 0, "Mutex destroy failed for package %s", p->name);
@@ -919,6 +1249,29 @@ sc_init (sc_MPI_Comm mpicomm,
   SC_GLOBAL_PRODUCTIONF ("%-*s %s\n", w, "LAPACK_LIBS", SC_LAPACK_LIBS);
   SC_GLOBAL_PRODUCTIONF ("%-*s %s\n", w, "FLIBS", SC_FLIBS);
 #endif
+
+#if defined(SC_ENABLE_MPI) && defined(SC_ENABLE_MPICOMMSHARED)
+  if (mpicomm != MPI_COMM_NULL) {
+    int                 mpiret;
+    MPI_Comm            intranode, internode;
+
+    /* compute the node comms by default */
+    sc_mpi_comm_attach_node_comms (mpicomm, 0);
+    sc_mpi_comm_get_node_comms (mpicomm, &intranode, &internode);
+    if (intranode == MPI_COMM_NULL) {
+      SC_GLOBAL_STATISTICS ("No shared memory node communicators\n");
+    }
+    else {
+      int                 intrasize;
+
+      mpiret = MPI_Comm_size (intranode, &intrasize);
+      SC_CHECK_MPI (mpiret);
+
+      SC_GLOBAL_STATISTICSF ("Shared memory node communicator size: %d\n",
+                             intrasize);
+    }
+  }
+#endif
 }
 
 void
@@ -927,6 +1280,10 @@ sc_finalize (void)
   int                 i;
   int                 retval;
 
+#if defined(SC_ENABLE_MPI) && defined(SC_ENABLE_MPICOMMSHARED)
+  sc_mpi_comm_detach_node_comms (sc_mpicomm);
+#endif
+
   /* sc_packages is static and thus initialized to all zeros */
   for (i = sc_num_packages_alloc - 1; i >= 0; --i)
     if (sc_packages[i].is_registered)
