@@ -3,6 +3,7 @@
   The SC Library provides support for parallel scientific applications.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
 
   The SC Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -43,6 +44,34 @@
 #define _sc_restrict restrict
 #endif
 
+/* test for gcc version without features.h */
+#define SC_CALC_VERSION(major,minor,patchlevel) \
+                       (((major) * 1000 + (minor)) * 1000 + (patchlevel))
+#ifdef __GNUC__
+#define SC_GCC_VERSION \
+        SC_CALC_VERSION(__GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__)
+#else
+#define SC_GCC_VERSION \
+        SC_CALC_VERSION (0, 0, 0)
+#endif
+
+/* use this feature macro, be minimally invasive */
+#ifdef SC_ENABLE_MEMALIGN
+/* we disable the system-provided functions for the time being */
+#ifdef SC_HAVE_ANY_MEMALIGN
+#undef SC_HAVE_ANY_MEMALIGN
+#endif
+/* if system-provided functions are needed, give them the prototype */
+#ifdef SC_HAVE_ANY_MEMALIGN
+#ifndef SC_HAVE_POSIX_MEMALIGN
+#ifdef SC_HAVE_ALIGNED_ALLOC
+#define _ISOC11_SOURCE
+#endif
+#endif
+#endif
+/* done with memalign macros */
+#endif
+
 /* use this in case mpi.h includes stdint.h */
 
 #ifndef __STDC_LIMIT_MACROS
@@ -76,13 +105,16 @@
 
 /* provide extern C defines */
 
-/* the hacks below enable semicolons after the SC_EXTERN_C_ macros */
+/* The hacks below enable semicolons after the SC_EXTERN_C_ macros
+ * and also take care of the different semantics of () / (...) */
 #ifdef __cplusplus
 #define SC_EXTERN_C_BEGIN       extern "C" { void sc_extern_c_hack_1 (void)
 #define SC_EXTERN_C_END                    } void sc_extern_c_hack_2 (void)
+#define SC_NOARGS               ...
 #else
 #define SC_EXTERN_C_BEGIN                    void sc_extern_c_hack_3 (void)
 #define SC_EXTERN_C_END                      void sc_extern_c_hack_4 (void)
+#define SC_NOARGS
 #endif
 
 /* this libsc header is always included */
@@ -207,7 +239,7 @@ void                SC_CHECK_ABORTF (int success, const char *fmt, ...)
 
 /* assertions, only enabled in debug mode */
 
-#ifdef SC_DEBUG
+#ifdef SC_ENABLE_DEBUG
 #define SC_ASSERT(c) SC_CHECK_ABORT ((c), "Assertion '" #c "'")
 #define SC_EXECUTE_ASSERT_FALSE(expression)                             \
   do { int _sc_i = (int) (expression);                                  \
@@ -235,6 +267,37 @@ void                SC_CHECK_ABORTF (int success, const char *fmt, ...)
 #define SC_STRDUP(s)                sc_strdup (sc_package_id, (s))
 #define SC_FREE(p)                  sc_free (sc_package_id, (p))
 
+/* macros for memory alignment */
+/* some copied from bfam: https://github.com/bfam/bfam */
+
+#define SC_ALIGN_UP(x,n) ( ((n) <= 0) ? (x) : ((x) + (n) - 1) / (n) * (n) )
+
+#if defined (__bgq__)
+#define SC_ARG_ALIGN(p,t,n) __alignx((n), (p))
+#elif defined (__ICC)
+#define SC_ARG_ALIGN(p,t,n) __assume_aligned((p), (n))
+#elif defined (__clang__)
+#define SC_ARG_ALIGN(p,t,n) SC_NOOP ()
+#elif defined (__GNUC__) || defined (__GNUG__)
+
+#if SC_GCC_VERSION >= SC_CALC_VERSION (4, 7, 0)
+#define SC_ARG_ALIGN(p,t,n) do {                              \
+  (p) = (t) __builtin_assume_aligned((void *) (p), (n));      \
+} while (0)
+#else
+#define SC_ARG_ALIGN(p,t,n) SC_NOOP ()
+#endif
+
+#else
+#define SC_ARG_ALIGN(p,t,n) SC_NOOP ()
+#endif
+
+#if (defined __GNUC__) || (defined __PGI) || (defined __IBMC__)
+#define SC_ATTR_ALIGN(n) __attribute__((aligned(n)))
+#else
+#define SC_ATTR_ALIGN(n)
+#endif
+
 /**
  * Sets n elements of a memory range to zero.
  * Assumes the pointer p is of the correct type.
@@ -299,7 +362,7 @@ void                SC_CHECK_ABORTF (int success, const char *fmt, ...)
 #ifdef SC_LOG_PRIORITY
 #define SC_LP_THRESHOLD SC_LOG_PRIORITY
 #else
-#ifdef SC_DEBUG
+#ifdef SC_ENABLE_DEBUG
 #define SC_LP_THRESHOLD SC_LP_TRACE
 #else
 #define SC_LP_THRESHOLD SC_LP_INFO
@@ -425,6 +488,7 @@ typedef void        (*sc_log_handler_t) (FILE * log_stream,
                                          const char *filename, int lineno,
                                          int package, int category,
                                          int priority, const char *msg);
+typedef void        (*sc_abort_handler_t) (void);
 
 /* memory allocation functions, will abort if out of memory */
 
@@ -455,6 +519,12 @@ void                sc_set_log_defaults (FILE * log_stream,
                                          sc_log_handler_t log_handler,
                                          int log_thresold);
 
+/** Controls the default SC abort behavior.
+ * \param [in] abort_handler Set default SC above handler (NULL selects
+ *                           builtin).  ***This function should not return!***
+ */
+void                sc_set_abort_handler (sc_abort_handler_t abort_handler);
+
 /** The central log function to be called by all packages.
  * Dispatches the log calls by package and filters by category and priority.
  * \param [in] package   Must be a registered package id or -1.
@@ -516,15 +586,62 @@ void                sc_abort_collective (const char *msg)
 int                 sc_package_register (sc_log_handler_t log_handler,
                                          int log_threshold,
                                          const char *name, const char *full);
+
+/** Query whether an identifier matches a registered package.
+ * \param [in] package_id       Only a non-negative id can be registered.
+ * \return                      True if and only if the package id is
+ *                              non-negative and package is registered.
+ */
 int                 sc_package_is_registered (int package_id);
 
+/** Acquire a pthread mutex lock.
+ * If configured without --enable-pthread, this function does nothing.
+ * This function must be followed with a matching \ref sc_package_unlock.
+ * \param [in] package_id       Either -1 for an undefined package or
+ *                              an id returned from \ref sc_package_register.
+ *                              Depending on the value, the appropriate mutex
+ *                              is chosen.  Thus, we may overlap locking calls
+ *                              with distinct package_id.
+ */
+void                sc_package_lock (int package_id);
+
+/** Release a pthread mutex lock.
+ * If configured without --enable-pthread, this function does nothing.
+ * This function must be follow a matching \ref sc_package_lock.
+ * \param [in] package_id       Either -1 for an undefined package or
+ *                              an id returned from \ref sc_package_register.
+ *                              Depending on the value, the appropriate mutex
+ *                              is chosen.  Thus, we may overlap locking calls
+ *                              with distinct package_id.
+ */
+void                sc_package_unlock (int package_id);
+
+/** Set the logging verbosity of a registered package.
+ * This can be called at any point in the program, any number of times.
+ * It can only lower the verbosity at and below the value of SC_LP_THRESHOLD.
+ * \param [in] package_id       Must be a registered package identifier.
+ */
+void                sc_package_set_verbosity (int package_id,
+                                              int log_priority);
+
+/** Set the unregister behavior of sc_package_unregister().
+ *
+ * \param[in] package_id    Must be -1 for the default package or
+ *                          the identifier of a registered package.
+ * \param[in] set_abort     True if sc_package_unregister() should abort if the
+ *                          number of allocs does not match the number of
+ *                          frees; false otherwise.
+ */
+void                sc_package_set_abort_alloc_mismatch (int package_id,
+                                                         int set_abort);
+
 /** Unregister a software package with SC.
  * This function must only be called after additional threads are finished.
  */
 void                sc_package_unregister (int package_id);
 
 /** Print a summary of all packages registered with SC.
- * Uses the SC_LP_GLOBAL log category which by default only prints on rank 0.
+ * Uses the SC_LC_GLOBAL log category which by default only prints on rank 0.
  * \param [in] log_priority     Priority passed to sc log functions.
  */
 void                sc_package_print_summary (int log_priority);
