@@ -3,6 +3,7 @@
   The SC Library provides support for parallel scientific applications.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
 
   The SC Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -47,13 +48,13 @@ sc_array_new (size_t elem_size)
 }
 
 sc_array_t         *
-sc_array_new_size (size_t elem_size, size_t elem_count)
+sc_array_new_count (size_t elem_size, size_t elem_count)
 {
   sc_array_t         *array;
 
   array = SC_ALLOC (sc_array_t, 1);
 
-  sc_array_init_size (array, elem_size, elem_count);
+  sc_array_init_count (array, elem_size, elem_count);
 
   return array;
 }
@@ -91,6 +92,16 @@ sc_array_destroy (sc_array_t * array)
   SC_FREE (array);
 }
 
+void
+sc_array_destroy_null (sc_array_t ** parray)
+{
+  SC_ASSERT (parray != NULL);
+  SC_ASSERT (*parray != NULL);
+
+  sc_array_destroy (*parray);
+  *parray = NULL;
+}
+
 void
 sc_array_init (sc_array_t * array, size_t elem_size)
 {
@@ -104,6 +115,12 @@ sc_array_init (sc_array_t * array, size_t elem_size)
 
 void
 sc_array_init_size (sc_array_t * array, size_t elem_size, size_t elem_count)
+{
+  sc_array_init_count (array, elem_size, elem_count);
+}
+
+void
+sc_array_init_count (sc_array_t * array, size_t elem_size, size_t elem_count)
 {
   SC_ASSERT (elem_size > 0);
 
@@ -137,6 +154,12 @@ sc_array_init_data (sc_array_t * view, void *base, size_t elem_size,
   view->array = (char *) base;
 }
 
+void
+sc_array_memset (sc_array_t * array, int c)
+{
+  memset (array->array, c, array->elem_count * array->elem_size);
+}
+
 void
 sc_array_reset (sc_array_t * array)
 {
@@ -156,19 +179,38 @@ sc_array_truncate (sc_array_t * array)
 
   array->elem_count = 0;
 
-#if SC_DEBUG
+#if SC_ENABLE_DEBUG
   SC_ASSERT (array->byte_alloc >= 0);
-  memset (array->array, -1, array->byte_alloc);
+  memset (array->array, (char) -1, array->byte_alloc);
 #endif
 }
 
+void
+sc_array_rewind (sc_array_t * array, size_t new_count)
+{
+  SC_ASSERT (array != NULL);
+  SC_ASSERT (array->elem_count >= new_count);
+
+  if (new_count == 0 && SC_ARRAY_IS_OWNER (array)) {
+    sc_array_reset (array);
+  }
+  else {
+    array->elem_count = new_count;
+  }
+}
+
 void
 sc_array_resize (sc_array_t * array, size_t new_count)
 {
   size_t              newoffs, roundup, newsize;
-#ifdef SC_DEBUG
-  size_t              oldoffs;
-  size_t              i, minoffs;
+#if !defined SC_ENABLE_USE_REALLOC || defined SC_ENABLE_DEBUG
+  size_t              oldoffs, minoffs;
+#endif
+#ifndef SC_ENABLE_USE_REALLOC
+  char               *ptr;
+#endif
+#ifdef SC_ENABLE_DEBUG
+  size_t              i;
 #endif
 
   if (!SC_ARRAY_IS_OWNER (array)) {
@@ -186,38 +228,53 @@ sc_array_resize (sc_array_t * array, size_t new_count)
     return;
   }
 
-#ifdef SC_DEBUG
+  /* Figure out how the array size will change */
+  newoffs = new_count * array->elem_size;
+#if defined SC_ENABLE_DEBUG || !defined SC_ENABLE_USE_REALLOC
   oldoffs = array->elem_count * array->elem_size;
+  minoffs = SC_MIN (oldoffs, newoffs);
 #endif
   array->elem_count = new_count;
-  newoffs = array->elem_count * array->elem_size;
   roundup = (size_t) SC_ROUNDUP2_64 (newoffs);
   SC_ASSERT (roundup >= newoffs && roundup <= 2 * newoffs);
 
   if (newoffs > (size_t) array->byte_alloc ||
       roundup < (size_t) array->byte_alloc) {
+    /* we will reallocate the array memory, either grow or shrink it */
     array->byte_alloc = (ssize_t) roundup;
   }
   else {
-#ifdef SC_DEBUG
+#ifdef SC_ENABLE_DEBUG
     if (newoffs < oldoffs) {
-      memset (array->array + newoffs, -1, oldoffs - newoffs);
+      memset (array->array + newoffs, (char) -1, oldoffs - newoffs);
     }
     for (i = oldoffs; i < newoffs; ++i) {
       SC_ASSERT (array->array[i] == (char) -1);
     }
 #endif
+    /* we keep the current allocation */
     return;
   }
+
+  /* byte_alloc is the size to be realloced to, it may be smaller than oldoffs */
   SC_ASSERT ((size_t) array->byte_alloc >= newoffs);
 
   newsize = (size_t) array->byte_alloc;
+#ifdef SC_ENABLE_USE_REALLOC
   array->array = SC_REALLOC (array->array, char, newsize);
+#else
+  ptr = SC_ALLOC (char, newsize);
+  if (minoffs > 0) {
+    /* avoid calling memcpy on less well supported corner cases */
+    memcpy (ptr, array->array, minoffs);
+  }
+  SC_FREE (array->array);
+  array->array = ptr;
+#endif
 
-#ifdef SC_DEBUG
-  minoffs = SC_MIN (oldoffs, newoffs);
+#ifdef SC_ENABLE_DEBUG
   SC_ASSERT (minoffs <= newsize);
-  memset (array->array + minoffs, -1, newsize - minoffs);
+  memset (array->array + minoffs, (char) -1, newsize - minoffs);
 #endif
 }
 
@@ -227,10 +284,46 @@ sc_array_copy (sc_array_t * dest, sc_array_t * src)
   SC_ASSERT (SC_ARRAY_IS_OWNER (dest));
   SC_ASSERT (dest->elem_size == src->elem_size);
 
+  /* always resize the destination array as documented */
   sc_array_resize (dest, src->elem_count);
+
+  if (src->elem_count == 0 || src->elem_size == 0) {
+    /* avoid calling memcpy on less well supported corner cases */
+    return;
+  }
   memcpy (dest->array, src->array, src->elem_count * src->elem_size);
 }
 
+void
+sc_array_copy_into (sc_array_t * dest, size_t dest_offset, sc_array_t * src)
+{
+  SC_ASSERT (dest->elem_size == src->elem_size);
+  SC_ASSERT (dest_offset + src->elem_count <= dest->elem_count);
+
+  if (src->elem_count == 0 || src->elem_size == 0) {
+    /* avoid calling memcpy on less well supported corner cases */
+    return;
+  }
+  memcpy (dest->array + dest_offset * dest->elem_size,
+          src->array, src->elem_count * src->elem_size);
+}
+
+void
+sc_array_move_part (sc_array_t * dest, size_t dest_offset,
+                    sc_array_t * src, size_t src_offset, size_t count)
+{
+  SC_ASSERT (dest->elem_size == src->elem_size);
+  SC_ASSERT (dest_offset + count <= dest->elem_count);
+  SC_ASSERT (src_offset + count <= src->elem_count);
+
+  if (count == 0 || src->elem_size == 0) {
+    /* avoid calling memmove on less well supported corner cases */
+    return;
+  }
+  memmove (dest->array + dest_offset * dest->elem_size,
+           src->array + src_offset * src->elem_size, count * src->elem_size);
+}
+
 void
 sc_array_sort (sc_array_t * array, int (*compar) (const void *, const void *))
 {
@@ -658,16 +751,123 @@ sc_array_pqueue_pop (sc_array_t * array, void *result,
   return swaps;
 }
 
+/* memory stamp routines */
+
+static void
+sc_mstamp_stamp (sc_mstamp_t * mst)
+{
+  SC_ASSERT (mst != NULL);
+  SC_ASSERT (mst->elem_size > 0);
+  SC_ASSERT (mst->stamp_size > 0);
+
+  /* make new stamp; the pointer is aligned to any builtin type */
+  mst->cur_snext = 0;
+  *(void **) sc_array_push (&mst->remember) =
+    mst->current = SC_ALLOC (char, mst->stamp_size);
+}
+
+void
+sc_mstamp_init (sc_mstamp_t * mst, size_t stamp_unit, size_t elem_size)
+{
+  SC_ASSERT (mst != NULL);
+
+  /* basic initialization */
+  memset (mst, 0, sizeof (sc_mstamp_t));
+  mst->elem_size = elem_size;
+  sc_array_init (&mst->remember, sizeof (void *));
+
+  /* how many items per stamp we use */
+  if (elem_size > 0) {
+    mst->per_stamp = stamp_unit / elem_size;
+    if (mst->per_stamp == 0) {
+      /* Each item uses more memory than we had specified for one stamp */
+      mst->per_stamp = 1;
+    }
+    mst->stamp_size = mst->per_stamp * elem_size;
+    sc_mstamp_stamp (mst);
+  }
+}
+
+void
+sc_mstamp_reset (sc_mstamp_t * mst)
+{
+  size_t              znum, zz;
+
+  SC_ASSERT (mst != NULL);
+
+  /* free all memory stamps we have created */
+  znum = mst->remember.elem_count;
+  for (zz = 0; zz < znum; zz++) {
+    SC_FREE (*(void **) sc_array_index (&mst->remember, zz));
+  }
+  sc_array_reset (&mst->remember);
+}
+
+void
+sc_mstamp_truncate (sc_mstamp_t * mst)
+{
+  /* free all memory in structure; the array mst->remember will be legal */
+  sc_mstamp_reset (mst);
+
+  /* we will use the container is if freshly initialized */
+  if (mst->elem_size > 0) {
+    sc_mstamp_stamp (mst);
+  }
+}
+
+void               *
+sc_mstamp_alloc (sc_mstamp_t * mst)
+{
+  void               *ret;
+
+  SC_ASSERT (mst != NULL);
+
+  if (mst->elem_size == 0) {
+    /* item size zero is legal */
+    return NULL;
+  }
+
+  /* we know that at least one item will fit */
+  SC_ASSERT (mst->current != NULL);
+  SC_ASSERT (mst->cur_snext < mst->per_stamp);
+  ret = mst->current + mst->cur_snext * mst->elem_size;
+
+  /* if this was the last item on the current stamp, we need a new one */
+  if (++mst->cur_snext == mst->per_stamp) {
+    sc_mstamp_stamp (mst);
+  }
+  return ret;
+}
+
+size_t
+sc_mstamp_memory_used (sc_mstamp_t * mst)
+{
+  size_t              s;
+
+  SC_ASSERT (mst != NULL);
+
+  s = sizeof (sc_mstamp_t);
+  s += mst->remember.elem_count * mst->stamp_size;
+  s += sc_array_memory_used (&mst->remember, 0);
+  return s;
+}
+
 /* mempool routines */
 
 size_t
 sc_mempool_memory_used (sc_mempool_t * mempool)
 {
   return sizeof (sc_mempool_t) +
+#ifdef SC_MEMPOOL_MSTAMP
+    sc_mstamp_memory_used (&mempool->mstamp) +
+#else
     obstack_memory_used (&mempool->obstack) +
+#endif
     sc_array_memory_used (&mempool->freed, 0);
 }
 
+#ifndef SC_MEMPOOL_MSTAMP
+
 static void        *
 sc_containers_malloc (size_t n)
 {
@@ -684,8 +884,34 @@ sc_containers_free (void *p)
 
 static void         (*obstack_chunk_free) (void *) = sc_containers_free;
 
-sc_mempool_t       *
-sc_mempool_new (size_t elem_size)
+#endif /* !SC_MEMPOOL_MSTAMP */
+
+/** This function is static; we do not like to expose _ext functions in libsc. */
+static void
+sc_mempool_init_ext (sc_mempool_t * mempool, size_t elem_size,
+                     int zero_and_persist)
+{
+  mempool->elem_size = elem_size;
+  mempool->elem_count = 0;
+  mempool->zero_and_persist = zero_and_persist;
+
+#ifdef SC_MEMPOOL_MSTAMP
+  sc_mstamp_init (&mempool->mstamp, 4096, elem_size);
+#else
+  obstack_init (&mempool->obstack);
+#endif
+  sc_array_init (&mempool->freed, sizeof (void *));
+}
+
+void
+sc_mempool_init (sc_mempool_t * mempool, size_t elem_size)
+{
+  sc_mempool_init_ext (mempool, elem_size, 0);
+}
+
+/** This function is static; we do not like to expose _ext functions in libsc. */
+static sc_mempool_t *
+sc_mempool_new_ext (size_t elem_size, int zero_and_persist)
 {
   sc_mempool_t       *mempool;
 
@@ -694,30 +920,61 @@ sc_mempool_new (size_t elem_size)
 
   mempool = SC_ALLOC (sc_mempool_t, 1);
 
-  mempool->elem_size = elem_size;
-  mempool->elem_count = 0;
-
-  obstack_init (&mempool->obstack);
-  sc_array_init (&mempool->freed, sizeof (void *));
+  sc_mempool_init_ext (mempool, elem_size, zero_and_persist);
 
   return mempool;
 }
 
+sc_mempool_t       *
+sc_mempool_new (size_t elem_size)
+{
+  return sc_mempool_new_ext (elem_size, 0);
+}
+
+sc_mempool_t       *
+sc_mempool_new_zero_and_persist (size_t elem_size)
+{
+  return sc_mempool_new_ext (elem_size, 1);
+}
+
+void
+sc_mempool_reset (sc_mempool_t * mempool)
+{
+  sc_array_reset (&mempool->freed);
+#ifdef SC_MEMPOOL_MSTAMP
+  sc_mstamp_reset (&mempool->mstamp);
+#else
+  obstack_free (&mempool->obstack, NULL);
+#endif
+}
+
 void
 sc_mempool_destroy (sc_mempool_t * mempool)
 {
-  sc_array_reset (&mempool->freed);
-  obstack_free (&mempool->obstack, NULL);
-
+  sc_mempool_reset (mempool);
   SC_FREE (mempool);
 }
 
+void
+sc_mempool_destroy_null (sc_mempool_t ** pmempool)
+{
+  SC_ASSERT (pmempool != NULL);
+  SC_ASSERT (*pmempool != NULL);
+
+  sc_mempool_destroy (*pmempool);
+  *pmempool = NULL;
+}
+
 void
 sc_mempool_truncate (sc_mempool_t * mempool)
 {
   sc_array_reset (&mempool->freed);
+#ifdef SC_MEMPOOL_MSTAMP
+  sc_mstamp_truncate (&mempool->mstamp);
+#else
   obstack_free (&mempool->obstack, NULL);
   obstack_init (&mempool->obstack);
+#endif
   mempool->elem_count = 0;
 }
 
@@ -807,7 +1064,7 @@ sc_list_unlink (sc_list_t * list)
   list->elem_count = 0;
 }
 
-void
+sc_link_t          *
 sc_list_prepend (sc_list_t * list, void *data)
 {
   sc_link_t          *lynk;
@@ -821,9 +1078,10 @@ sc_list_prepend (sc_list_t * list, void *data)
   }
 
   ++list->elem_count;
+  return lynk;
 }
 
-void
+sc_link_t          *
 sc_list_append (sc_list_t * list, void *data)
 {
   sc_link_t          *lynk;
@@ -840,9 +1098,10 @@ sc_list_append (sc_list_t * list, void *data)
   list->last = lynk;
 
   ++list->elem_count;
+  return lynk;
 }
 
-void
+sc_link_t          *
 sc_list_insert (sc_list_t * list, sc_link_t * pred, void *data)
 {
   sc_link_t          *lynk;
@@ -858,6 +1117,7 @@ sc_list_insert (sc_list_t * list, sc_link_t * pred, void *data)
   }
 
   ++list->elem_count;
+  return lynk;
 }
 
 void               *
@@ -870,6 +1130,7 @@ sc_list_remove (sc_list_t * list, sc_link_t * pred)
     return sc_list_pop (list);
   }
 
+  SC_ASSERT (list->first != NULL && list->last != NULL);
   SC_ASSERT (pred->next != NULL);
 
   lynk = pred->next;
@@ -890,7 +1151,7 @@ sc_list_pop (sc_list_t * list)
   sc_link_t          *lynk;
   void               *data;
 
-  SC_ASSERT (list->first != NULL);
+  SC_ASSERT (list->first != NULL && list->last != NULL);
 
   lynk = list->first;
   list->first = lynk->next;
@@ -1001,7 +1262,7 @@ sc_hash_maybe_resize (sc_hash_t * hash)
       /* insert data into new slot list */
       j = hash->hash_fn (lynk->data, hash->user_data) % new_size;
       new_list = (sc_list_t *) sc_array_index (new_slots, j);
-      sc_list_prepend (new_list, lynk->data);
+      (void) sc_list_prepend (new_list, lynk->data);
       ++new_count;
 
       /* remove old list element */
@@ -1074,6 +1335,16 @@ sc_hash_destroy (sc_hash_t * hash)
   SC_FREE (hash);
 }
 
+void
+sc_hash_destroy_null (sc_hash_t ** phash)
+{
+  SC_ASSERT (phash != NULL);
+  SC_ASSERT (*phash != NULL);
+
+  sc_hash_destroy (*phash);
+  *phash = NULL;
+}
+
 void
 sc_hash_truncate (sc_hash_t * hash)
 {
@@ -1174,7 +1445,7 @@ sc_hash_insert_unique (sc_hash_t * hash, void *v, void ***found)
   }
 
   /* append new object to the list */
-  sc_list_append (list, v);
+  (void) sc_list_append (list, v);
   if (found != NULL) {
     *found = &list->last->data;
   }
