@@ -3,6 +3,7 @@
   The SC Library provides support for parallel scientific applications.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
 
   The SC Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -37,7 +38,14 @@
  * \ingroup sc
  */
 
+/** We are using sc_mstamp_t instead of GNU obstack in sc_mempool_t. */
+#define SC_MEMPOOL_MSTAMP
+
+#ifndef SC_MEMPOOL_MSTAMP
 #include <sc_obstack.h>
+#else
+#include <sc.h>
+#endif
 
 SC_EXTERN_C_BEGIN;
 
@@ -80,14 +88,13 @@ typedef int         (*sc_equal_function_t) (const void *v1,
  */
 typedef int         (*sc_hash_foreach_t) (void **v, const void *u);
 
-/** The sc_array object provides a large array of equal-size elements.
- * The array can be resized.
- * Elements are accessed by their 0-based index, their address may change.
- * The size (== elem_count) of the array can be changed by array_resize.
- * Elements can be sorted with array_sort.
- * If the array is sorted elements can be binary searched with array_bsearch.
- * A priority queue is implemented with pqueue_add and pqueue_pop.
- * Use sort and search whenever possible, they are faster than the pqueue.
+/** The sc_array object provides a dynamic array of equal-size elements.
+ * Elements are accessed by their 0-based index.  Their address may change.
+ * The number of elements (== elem_count) of the array can be changed by 
+ * \ref sc_array_resize and \ref sc_array_rewind.
+ * Elements can be sorted with \ref sc_array_sort.
+ * If the array is sorted, it can be searched with \ref sc_array_bsearch.
+ * A priority queue is implemented with pqueue_add and pqueue_pop (untested).
  */
 typedef struct sc_array
 {
@@ -131,7 +138,10 @@ sc_array_t         *sc_array_new (size_t elem_size);
  * \return                  Return an allocated array
  *                          with allocated but uninitialized elements.
  */
-sc_array_t         *sc_array_new_size (size_t elem_size, size_t elem_count);
+sc_array_t         *sc_array_new_count (size_t elem_size, size_t elem_count);
+
+/** Deprecated: use \ref sc_array_new_count. */
+#define sc_array_new_size(s,c) (sc_array_new_count ((s), (c)))
 
 /** Creates a new view of an existing sc_array_t.
  * \param [in] array    The array must not be resized while view is alive.
@@ -157,6 +167,12 @@ sc_array_t         *sc_array_new_data (void *base,
  */
 void                sc_array_destroy (sc_array_t * array);
 
+/** Destroys an array structure and sets the pointer to NULL.
+ * \param [in,out] parray       Pointer to address of array to be destroyed.
+ *                              On output, *parray is NULL.
+ */
+void                sc_array_destroy_null (sc_array_t ** parray);
+
 /** Initializes an already allocated (or static) array structure.
  * \param [in,out]  array       Array structure to be initialized.
  * \param [in] elem_size        Size of one array element in bytes.
@@ -165,6 +181,7 @@ void                sc_array_init (sc_array_t * array, size_t elem_size);
 
 /** Initializes an already allocated (or static) array structure
  * and allocates a given number of elements.
+ * Deprecated: use \ref sc_array_init_count.
  * \param [in,out]  array       Array structure to be initialized.
  * \param [in] elem_size        Size of one array element in bytes.
  * \param [in] elem_count       Number of initial array elements.
@@ -172,32 +189,57 @@ void                sc_array_init (sc_array_t * array, size_t elem_size);
 void                sc_array_init_size (sc_array_t * array,
                                         size_t elem_size, size_t elem_count);
 
+/** Initializes an already allocated (or static) array structure
+ * and allocates a given number of elements.
+ * This function supersedes \ref sc_array_init_size.
+ * \param [in,out]  array       Array structure to be initialized.
+ * \param [in] elem_size        Size of one array element in bytes.
+ * \param [in] elem_count       Number of initial array elements.
+ */
+void                sc_array_init_count (sc_array_t * array,
+                                         size_t elem_size, size_t elem_count);
+
 /** Initializes an already allocated (or static) view from existing sc_array_t.
+ * The array view returned does not require sc_array_reset (doesn't hurt though).
  * \param [in,out] view  Array structure to be initialized.
  * \param [in] array     The array must not be resized while view is alive.
  * \param [in] offset    The offset of the viewed section in element units.
  *                       This offset cannot be changed until the view is reset.
  * \param [in] length    The length of the view in element units.
  *                       The view cannot be resized to exceed this length.
+ *                       It is not necessary to call sc_array_reset later.
  */
 void                sc_array_init_view (sc_array_t * view, sc_array_t * array,
                                         size_t offset, size_t length);
 
 /** Initializes an already allocated (or static) view from given plain C data.
+ * The array view returned does not require sc_array_reset (doesn't hurt though).
  * \param [in,out] view     Array structure to be initialized.
  * \param [in] base         The data must not be moved while view is alive.
  * \param [in] elem_size    Size of one array element in bytes.
  * \param [in] elem_count   The length of the view in element units.
  *                          The view cannot be resized to exceed this length.
+ *                          It is not necessary to call sc_array_reset later.
  */
 void                sc_array_init_data (sc_array_t * view, void *base,
                                         size_t elem_size, size_t elem_count);
 
+/** Run memset on the array storage.
+ * We pass the character to memset unchanged.  Thus, care must be taken when
+ * setting values below -1 or above 127, just as with standard memset (3).
+ * \param [in,out] array    This array's storage will be overwritten.
+ * \param [in] c            Character to overwrite every byte with.
+ */
+void                sc_array_memset (sc_array_t * array, int c);
+
 /** Sets the array count to zero and frees all elements.
  * This function turns a view into a newly initialized array.
  * \param [in,out]  array       Array structure to be reset.
  * \note Calling sc_array_init, then any array operations,
  *       then sc_array_reset is memory neutral.
+ *       As an exception, the two functions sc_array_init_view and
+ *       sc_array_init_data do not require a subsequent call to sc_array_reset.
+ *       Regardless, it is legal to call sc_array_reset anyway.
  */
 void                sc_array_reset (sc_array_t * array);
 
@@ -210,22 +252,69 @@ void                sc_array_reset (sc_array_t * array);
  */
 void                sc_array_truncate (sc_array_t * array);
 
+/** Shorten an array without reallocating it.
+ * \param [in,out] array    The element count of this array is modified.
+ * \param [in] new_count    Must be less or equal than the \b array's count.
+ *                          If it is less, the number of elements in the
+ *                          array is reduced without reallocating memory.
+ *                          The exception is a \b new_count of zero
+ *                          specified for an array that is not a view:
+ *                          In this case \ref sc_array_reset is equivalent.
+ */
+void                sc_array_rewind (sc_array_t * array, size_t new_count);
+
 /** Sets the element count to new_count.
- * If this a view, new_count cannot be greater than the elem_count of
- * the view when it was created.  The original offset of the view cannot be
- * changed.
- * If this is an array, reallocation takes place only occasionally, so
- * this function is usually fast.
+ * If the array is not a view, reallocation takes place occasionally.
+ * If the array is a view, new_count must not be greater than the element
+ * count of the view when it was created.  The original offset of the view
+ * cannot be changed.
+ * \param [in,out] array    The element count and address is modified.
+ * \param [in] new_count    New element count of the array.
+ *                          If it is zero and the array is not a view,
+ *                          the effect equals \ref sc_array_reset.
  */
 void                sc_array_resize (sc_array_t * array, size_t new_count);
 
-/** Copy the contents of an array into another.
+/** Copy the contents of one array into another.
  * Both arrays must have equal element sizes.
- * \param [in] dest Array (not a view) will be resized and get new data.
- * \param [in] src  Array used as source of new data, will not be changed.
+ * The source array may be a view.
+ * We use memcpy (3):  If the two arrays overlap, results are undefined.
+ * \param [in] dest     Array (not a view) will be resized and get new data.
+ * \param [in] src      Array used as source of new data, will not be changed.
  */
 void                sc_array_copy (sc_array_t * dest, sc_array_t * src);
 
+/** Copy the contents of one array into some portion of another.
+ * Both arrays must have equal element sizes.
+ * Either array may be a view.  The destination array must be large enough.
+ * We use memcpy (3):  If the two arrays overlap, results are undefined.
+ * \param [in] dest     Array will be written into.  Its element count must
+ *                      be at least \b dest_offset + \b src->elem_count.
+ * \param [in] dest_offset  First index in \b dest array to be overwritten.
+ *                      As every index, it refers to elements, not bytes.
+ * \param [in] src      Array used as source of new data, will not be changed.
+ */
+void                sc_array_copy_into (sc_array_t * dest, size_t dest_offset,
+                                        sc_array_t * src);
+
+/** Copy part of one array into another using memmove (3).
+ * Both arrays must have equal element sizes.
+ * Either array may be a view.  The destination array must be large enough.
+ * We use memmove (3):  The two arrays may overlap.
+ * \param [in] dest     Array will be written into.  Its element count must
+ *                      be at least \b dest_offset + \b count.
+ * \param [in] dest_offset  First index in \b dest array to be overwritten.
+ *                      As every index, it refers to elements, not bytes.
+ * \param [in] src      Array will be read from.  Its element count must
+ *                      be at least \b src_offset + \b count.
+ * \param [in] src_offset   First index in \b src array to be used.
+ *                      As every index, it refers to elements, not bytes.
+ * \param [in] count    Number of entries copied.
+ */
+void                sc_array_move_part (sc_array_t * dest, size_t dest_offset,
+                                        sc_array_t * src, size_t src_offset,
+                                        size_t count);
+
 /** Sorts the array in ascending order wrt. the comparison function.
  * \param [in] array    The array to sort.
  * \param [in] compar   The comparison function to be used.
@@ -361,7 +450,9 @@ size_t              sc_array_pqueue_pop (sc_array_t * array,
                                                         const void *));
 
 /** Returns a pointer to an array element.
+ * \param [in] array Valid array.
  * \param [in] index needs to be in [0]..[elem_count-1].
+ * \return           Pointer to the indexed array element.
  */
 /*@unused@*/
 static inline void *
@@ -369,7 +460,23 @@ sc_array_index (sc_array_t * array, size_t iz)
 {
   SC_ASSERT (iz < array->elem_count);
 
-  return (void *) (array->array + (array->elem_size * iz));
+  return (void *) (array->array + array->elem_size * iz);
+}
+
+/** Returns a pointer to an array element or NULL at the array's end.
+ * \param [in] array Valid array.
+ * \param [in] index needs to be in [0]..[elem_count].
+ * \return           Pointer to the indexed array element or
+ *                   NULL if the specified index is elem_count.
+ */
+/*@unused@*/
+static inline void *
+sc_array_index_null (sc_array_t * array, size_t iz)
+{
+  SC_ASSERT (iz <= array->elem_count);
+
+  return iz == array->elem_count ? NULL :
+    (void *) (array->array + array->elem_size * iz);
 }
 
 /** Returns a pointer to an array element indexed by a plain int.
@@ -427,15 +534,16 @@ sc_array_index_int16 (sc_array_t * array, int16_t i16)
 static inline       size_t
 sc_array_position (sc_array_t * array, void *element)
 {
-  size_t              position;
+  ptrdiff_t           position;
 
   SC_ASSERT (array->array <= (char *) element);
-  SC_ASSERT (((char *) element - array->array) % array->elem_size == 0);
+  SC_ASSERT (((char *) element - array->array) %
+             (ptrdiff_t) array->elem_size == 0);
 
-  position = ((char *) element - array->array) / array->elem_size;
-  SC_ASSERT (position < array->elem_count);
+  position = ((char *) element - array->array) / (ptrdiff_t) array->elem_size;
+  SC_ASSERT (0 <= position && position < (ptrdiff_t) array->elem_count);
 
-  return position;
+  return (size_t) position;
 }
 
 /** Remove the last element from an array and return a pointer to it.
@@ -487,20 +595,98 @@ sc_array_push (sc_array_t * array)
   return sc_array_push_count (array, 1);
 }
 
+/** A data container to create memory items of the same size.
+ * Allocations are bundled so it's fast for small memory sizes.
+ * The items created will remain valid until the container is destroyed.
+ * There is no option to return an item to the container.
+ * See \ref sc_mempool_t for that purpose.
+ */
+typedef struct sc_mstamp
+{
+  size_t              elem_size;   /**< Input parameter: size per item */
+  size_t              per_stamp;   /**< Number of items per stamp */
+  size_t              stamp_size;  /**< Bytes allocated in a stamp */
+  size_t              cur_snext;   /**< Next number within a stamp */
+  char               *current;     /**< Memory of current stamp */
+  sc_array_t          remember;    /**< Collects all stamps */
+}
+sc_mstamp_t;
+
+/** Initialize a memory stamp container.
+ * We provide allocation of fixed-size memory items
+ * without allocating new memory in every request.
+ * Instead we block the allocations in what we call a stamp of multiple items.
+ * Even if no allocations are done, the container's internal memory
+ * must be freed eventually by \ref sc_mstamp_reset.
+ *
+ * \param [in,out] mst          Legal pointer to a stamp structure.
+ * \param [in] stamp_unit       Size of each memory block that we allocate.
+ *                              If it is larger than the element size,
+ *                              we may place more than one element in it.
+ *                              Passing 0 is legal and forces
+ *                              stamps that hold one item each.
+ * \param [in] elem_size        Size of each item.
+ *                              Passing 0 is legal.  In that case,
+ *                              \ref sc_mstamp_alloc returns NULL.
+ */
+void                sc_mstamp_init (sc_mstamp_t * mst,
+                                    size_t stamp_unit, size_t elem_size);
+
+/** Free all memory in a stamp structure and all items previously returned.
+ * \param [in,out]              Properly initialized stamp container.
+ *                              On output, the structure is undefined.
+ */
+void                sc_mstamp_reset (sc_mstamp_t * mst);
+
+/** Free all memory in a stamp structure and initialize it anew.
+ * Equivalent to calling \ref sc_mstamp_reset followed by
+ *                       \ref sc_mstamp_init with the same
+ *                            stamp_unit and elem_size.
+ *
+ * \param [in,out]              Properly initialized stamp container.
+ *                              On output, its elements have been freed
+ *                              and it is ready for further use.
+ */
+void                sc_mstamp_truncate (sc_mstamp_t * mst);
+
+/** Return a new item.
+ * The memory returned will stay legal
+ * until container is destroyed or truncated.
+ * \param [in,out]              Properly initialized stamp container.
+ * \return                      Pointer to an item ready to use.
+ *                              Legal until \ref sc_stamp_destroy or
+ *                              \ref sc_stamp_truncate is called on mst.
+ */
+void               *sc_mstamp_alloc (sc_mstamp_t * mst);
+
+/** Return memory size in bytes of all data allocated in the container.
+ * \param [in]                  Properly initialized stamp container.
+ * \return                      Total container memory size in bytes.
+ */
+size_t              sc_mstamp_memory_used (sc_mstamp_t * mst);
+
 /** The sc_mempool object provides a large pool of equal-size elements.
  * The pool grows dynamically for element allocation.
  * Elements are referenced by their address which never changes.
  * Elements can be freed (that is, returned to the pool)
  *    and are transparently reused.
+ * If the zero_and_persist option is selected, new elements are initialized to
+ * all zeros on creation, and the contents of an element are not touched
+ * between freeing and re-returning it.
  */
 typedef struct sc_mempool
 {
   /* interface variables */
   size_t              elem_size;        /**< size of a single element */
   size_t              elem_count;       /**< number of valid elements */
+  int                 zero_and_persist; /**< Boolean; is set in constructor. */
 
   /* implementation variables */
+#ifdef SC_MEMPOOL_MSTAMP
+  sc_mstamp_t         mstamp;   /**< our own obstack replacement */
+#else
   struct obstack      obstack;  /**< holds the allocated elements */
+#endif
   sc_array_t          freed;    /**< buffers the freed elements */
 }
 sc_mempool_t;
@@ -511,17 +697,41 @@ sc_mempool_t;
  */
 size_t              sc_mempool_memory_used (sc_mempool_t * mempool);
 
-/** Creates a new mempool structure.
+/** Creates a new mempool structure with the zero_and_persist option off.
+ * The contents of any elements returned by sc_mempool_alloc are undefined.
  * \param [in] elem_size  Size of one element in bytes.
  * \return Returns an allocated and initialized memory pool.
  */
 sc_mempool_t       *sc_mempool_new (size_t elem_size);
 
-/** Destroys a mempool structure.
+/** Creates a new mempool structure with the zero_and_persist option on.
+ * The memory of newly created elements is zero'd out, and the contents of an
+ * element are not touched between freeing and re-returning it.
+ * \param [in] elem_size  Size of one element in bytes.
+ * \return Returns an allocated and initialized memory pool.
+ */
+sc_mempool_t       *sc_mempool_new_zero_and_persist (size_t elem_size);
+
+/** Same as sc_mempool_new, but for an already allocated sc_mempool_t pointer. */
+void                sc_mempool_init (sc_mempool_t * mempool,
+                                     size_t elem_size);
+
+/** Destroy a mempool structure.
  * All elements that are still in use are invalidated.
+ * \param [in,out] mempool      Its memory is freed.
  */
 void                sc_mempool_destroy (sc_mempool_t * mempool);
 
+/** Destroy a mempool structure.
+ * All elements that are still in use are invalidated.
+ * \param [in,out] pmempool     Address of pointer to memory pool.
+ *                              Its memory is freed, pointer is NULLed.
+ */
+void                sc_mempool_destroy_null (sc_mempool_t ** pmempool);
+
+/** Same as sc_mempool_destroy, but does not free the pointer */
+void                sc_mempool_reset (sc_mempool_t * mempool);
+
 /** Invalidates all previously returned pointers, resets count to 0.
  */
 void                sc_mempool_truncate (sc_mempool_t * mempool);
@@ -543,11 +753,20 @@ sc_mempool_alloc (sc_mempool_t * mempool)
     ret = *(void **) sc_array_pop (freed);
   }
   else {
+#ifdef SC_MEMPOOL_MSTAMP
+    ret = sc_mstamp_alloc (&mempool->mstamp);
+#else
     ret = obstack_alloc (&mempool->obstack, (int) mempool->elem_size);
+#endif
+    if (mempool->zero_and_persist) {
+      memset (ret, 0, mempool->elem_size);
+    }
   }
 
-#ifdef SC_DEBUG
-  memset (ret, -1, mempool->elem_size);
+#ifdef SC_ENABLE_DEBUG
+  if (!mempool->zero_and_persist) {
+    memset (ret, -1, mempool->elem_size);
+  }
 #endif
 
   return ret;
@@ -564,8 +783,10 @@ sc_mempool_free (sc_mempool_t * mempool, void *elem)
 
   SC_ASSERT (mempool->elem_count > 0);
 
-#ifdef SC_DEBUG
-  memset (elem, -1, mempool->elem_size);
+#ifdef SC_ENABLE_DEBUG
+  if (!mempool->zero_and_persist) {
+    memset (elem, -1, mempool->elem_size);
+  }
 #endif
 
   --mempool->elem_count;
@@ -597,7 +818,7 @@ typedef struct sc_list
 }
 sc_list_t;
 
-/** Calculate the memory used by a list.
+/** Calculate the total memory used by a list.
  * \param [in] list        The list.
  * \param [in] is_dynamic  True if created with sc_list_new,
  *                         false if initialized with sc_list_init
@@ -605,52 +826,73 @@ sc_list_t;
  */
 size_t              sc_list_memory_used (sc_list_t * list, int is_dynamic);
 
-/** Allocate a linked list structure.
- * \param [in] allocator Memory allocator for sc_link_t, can be NULL.
+/** Allocate a new, empty linked list.
+ * \param [in] allocator    Memory allocator for sc_link_t, can be NULL
+ *                          in which case an internal allocator is created.
+ * \return                  Pointer to a newly allocated, empty list object.
  */
 sc_list_t          *sc_list_new (sc_mempool_t * allocator);
 
 /** Destroy a linked list structure in O(N).
+ * \param [in,out] list     All memory allocated for this list is freed.
  * \note If allocator was provided in sc_list_new, it will not be destroyed.
  */
 void                sc_list_destroy (sc_list_t * list);
 
-/** Initializes an already allocated list structure.
+/** Initialize a list object with an external link allocator.
  * \param [in,out]  list       List structure to be initialized.
- * \param [in]      allocator  External memory allocator for sc_link_t.
+ * \param [in]      allocator  External memory allocator for sc_link_t,
+ *                             which must exist already.
  */
 void                sc_list_init (sc_list_t * list, sc_mempool_t * allocator);
 
-/** Removes all elements from a list in O(N).
- * \param [in,out]  list       List structure to be resetted.
+/** Remove all elements from a list in O(N).
+ * \param [in,out]  list       List structure to be emptied.
  * \note Calling sc_list_init, then any list operations,
  *       then sc_list_reset is memory neutral.
  */
 void                sc_list_reset (sc_list_t * list);
 
-/** Unliks all list elements without returning them to the mempool.
- * This runs in O(1) but is dangerous because of potential memory leaks.
+/** Unlink all list elements without returning them to the mempool.
+ * This runs in O(1) but is dangerous because the link memory stays alive.
  * \param [in,out]  list       List structure to be unlinked.
  */
 void                sc_list_unlink (sc_list_t * list);
 
-void                sc_list_prepend (sc_list_t * list, void *data);
-void                sc_list_append (sc_list_t * list, void *data);
-
-/** Insert an element after a given position.
- * \param [in] pred The predecessor of the element to be inserted.
+/** Insert a list element at the beginning of the list.
+ * \param [in,out] list     Valid list object.
+ * \param [in] data         A new link is created holding this data.
+ * \return                  The link that has been created for data.
  */
-void                sc_list_insert (sc_list_t * list,
+sc_link_t          *sc_list_prepend (sc_list_t * list, void *data);
+
+/** Insert a list element at the end of the list.
+ * \param [in,out] list     Valid list object.
+ * \param [in] data         A new link is created holding this data.
+ * \return                  The link that has been created for data.
+ */
+sc_link_t          *sc_list_append (sc_list_t * list, void *data);
+
+/** Insert an element after a given list position.
+ * \param [in,out] list     Valid list object.
+ * \param [in,out] pred     The predecessor of the element to be inserted.
+ * \param [in] data         A new link is created holding this data.
+ * \return                  The link that has been created for data.
+ */
+sc_link_t          *sc_list_insert (sc_list_t * list,
                                     sc_link_t * pred, void *data);
 
-/** Remove an element after a given position.
+/** Remove an element after a given list position.
+ * \param [in,out] list     Valid, non-empty list object.
  * \param [in] pred  The predecessor of the element to be removed.
-                     If \a pred == NULL, the first element is removed.
- * \return Returns the data of the removed element.
+ *                   If \a pred == NULL, the first element is removed,
+ *                   which is equivalent to calling sc_list_pop (list).
+ * \return           The data of the removed and freed link.
  */
 void               *sc_list_remove (sc_list_t * list, sc_link_t * pred);
 
 /** Remove an element from the front of the list.
+ * \param [in,out] list     Valid, non-empty list object.
  * \return Returns the data of the removed first list element.
  */
 void               *sc_list_pop (sc_list_t * list);
@@ -706,6 +948,13 @@ sc_hash_t          *sc_hash_new (sc_hash_function_t hash_fn,
  */
 void                sc_hash_destroy (sc_hash_t * hash);
 
+/** Destroy a hash table and set its pointer to NULL.
+ * Destruction is done using \ref sc_hash_destroy.
+ * \param [in,out] phash        Address of pointer to hash table.
+ *                              On output, pointer is NULLed.
+ */
+void                sc_hash_destroy_null (sc_hash_t ** phash);
+
 /** Remove all entries from a hash table in O(N).
  *
  * If the allocator is owned, it calls sc_hash_unlink and sc_mempool_truncate.
