@@ -3,6 +3,7 @@
   The SC Library provides support for parallel scientific applications.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
 
   The SC Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -27,7 +28,7 @@ const char          sc_jobzchar[] = { 'N', 'V', '?' };
 #ifndef SC_WITH_LAPACK
 
 int
-sc_lapack_nonimplemented ()
+sc_lapack_nonimplemented (SC_NOARGS)
 {
   SC_ABORT ("LAPACK not compiled in this configuration");
   return 0;
