@@ -3,6 +3,7 @@
   The SC Library provides support for parallel scientific applications.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
 
   The SC Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -26,8 +27,54 @@
 
 #include <errno.h>
 
+typedef enum
+{
+  SC_OPTION_SWITCH,
+  SC_OPTION_BOOL,
+  SC_OPTION_INT,
+  SC_OPTION_SIZE_T,
+  SC_OPTION_DOUBLE,
+  SC_OPTION_STRING,
+  SC_OPTION_INIFILE,
+  SC_OPTION_CALLBACK,
+  SC_OPTION_KEYVALUE
+}
+sc_option_type_t;
+
+typedef struct
+{
+  sc_option_type_t    opt_type;
+  int                 opt_char;
+  const char         *opt_name;
+  void               *opt_var;
+  void                (*opt_fn) (void);
+  int                 has_arg;
+  int                 called;           /**< set to 0 and ignored */
+  const char         *help_string;
+  char               *string_value;     /**< set on call but ignored */
+  void               *user_data;
+}
+sc_option_item_t;
+
+struct sc_options
+{
+  char                program_path[BUFSIZ];
+  const char         *program_name;
+  sc_array_t         *option_items;
+  int                 space_type;
+  int                 space_help;
+  int                 args_alloced;
+  int                 first_arg;
+  int                 argc;
+  char              **argv;
+  sc_array_t         *subopt_names;
+};
+
 static char        *sc_iniparser_invalid_key = (char *) -1;
 
+static const int    sc_options_space_type = 20;
+static const int    sc_options_space_help = 32;
+
 static int
 sc_iniparser_getint (dictionary * d, const char *key, int notfound,
                      int *iserror)
@@ -139,11 +186,14 @@ sc_options_new (const char *program_path)
   opt->argc = 0;
   opt->argv = NULL;
 
+  /* set default spacing for printing option summary */
+  sc_options_set_spacing (opt, -1, -1);
+
   return opt;
 }
 
-void
-sc_options_destroy (sc_options_t * opt)
+static void
+sc_options_destroy_internal (sc_options_t * opt, int deep)
 {
   size_t              iz;
   sc_array_t         *items = opt->option_items;
@@ -154,6 +204,9 @@ sc_options_destroy (sc_options_t * opt)
 
   for (iz = 0; iz < count; ++iz) {
     item = (sc_option_item_t *) sc_array_index (items, iz);
+    if (deep && item->opt_type == SC_OPTION_KEYVALUE) {
+      sc_keyvalue_destroy ((sc_keyvalue_t *) item->user_data);
+    }
     SC_FREE (item->string_value);
   }
 
@@ -170,6 +223,27 @@ sc_options_destroy (sc_options_t * opt)
   SC_FREE (opt);
 }
 
+void
+sc_options_destroy_deep (sc_options_t * opt)
+{
+  sc_options_destroy_internal (opt, 1);
+}
+
+void
+sc_options_destroy (sc_options_t * opt)
+{
+  sc_options_destroy_internal (opt, 0);
+}
+
+void
+sc_options_set_spacing (sc_options_t * opt, int space_type, int space_help)
+{
+  SC_ASSERT (opt != NULL);
+
+  opt->space_type = space_type < 0 ? sc_options_space_type : space_type;
+  opt->space_help = space_help < 0 ? sc_options_space_help : space_help;
+}
+
 void
 sc_options_add_switch (sc_options_t * opt, int opt_char,
                        const char *opt_name,
@@ -374,6 +448,39 @@ sc_options_add_callback (sc_options_t * opt, int opt_char,
   item->user_data = data;
 }
 
+void
+sc_options_add_keyvalue (sc_options_t * opt,
+                         int opt_char, const char *opt_name,
+                         int *variable, const char *init_value,
+                         sc_keyvalue_t * keyvalue, const char *help_string)
+{
+  sc_option_item_t   *item;
+
+  SC_ASSERT (opt_char != '\0' || opt_name != NULL);
+  SC_ASSERT (opt_name == NULL || opt_name[0] != '-');
+
+  /* we do not accept an invalid default value */
+  SC_ASSERT (variable != NULL);
+  SC_ASSERT (init_value != NULL);
+  SC_ASSERT (keyvalue != NULL);
+
+  item = (sc_option_item_t *) sc_array_push (opt->option_items);
+
+  item->opt_type = SC_OPTION_KEYVALUE;
+  item->opt_char = opt_char;
+  item->opt_name = opt_name;
+  item->opt_var = variable;
+  item->opt_fn = NULL;
+  item->has_arg = 1;
+  item->called = 0;
+  item->help_string = help_string;
+  item->user_data = keyvalue;
+
+  /* we expect that the key points to a valid integer entry by design */
+  *variable = sc_keyvalue_get_int_check (keyvalue, init_value, NULL);
+  item->string_value = SC_STRDUP (init_value);
+}
+
 void
 sc_options_add_suboptions (sc_options_t * opt,
                            sc_options_t * subopt, const char *prefix)
@@ -434,6 +541,13 @@ sc_options_add_suboptions (sc_options_t * opt,
                                (sc_options_callback_t) item->opt_fn,
                                item->user_data, item->help_string);
       break;
+    case SC_OPTION_KEYVALUE:
+      SC_ASSERT (item->string_value != NULL);
+      sc_options_add_keyvalue (opt, '\0', *name,
+                               (int *) item->opt_var, item->string_value,
+                               (sc_keyvalue_t *) item->user_data,
+                               item->help_string);
+      break;
     default:
       SC_ABORT_NOT_REACHED ();
     }
@@ -449,8 +563,8 @@ sc_options_print_usage (int package_id, int log_priority,
   sc_array_t         *items = opt->option_items;
   size_t              count = items->elem_count;
   sc_option_item_t   *item;
-  const char         *provide_short;
-  const char         *provide_long;
+  const char         *provide;
+  const char         *separator;
   char                outbuf[BUFSIZ];
   char               *copy, *tok;
 
@@ -464,65 +578,68 @@ sc_options_print_usage (int package_id, int log_priority,
 
   for (iz = 0; iz < count; ++iz) {
     item = (sc_option_item_t *) sc_array_index (items, iz);
-    provide_short = "";
-    provide_long = "";
+    provide = "";
     switch (item->opt_type) {
     case SC_OPTION_SWITCH:
       break;
     case SC_OPTION_BOOL:
-      provide_short = " [0fFnN1tTyY]";
-      provide_long = "[=0fFnN1tTyY]";
+      provide = "[0fFnN1tTyY]";
       break;
     case SC_OPTION_INT:
-      provide_short = " <INT>";
-      provide_long = "=<INT>";
+      provide = "<INT>";
       break;
     case SC_OPTION_SIZE_T:
-      provide_short = " <SIZE_T>";
-      provide_long = "=<SIZE_T>";
+      provide = "<SIZE_T>";
       break;
     case SC_OPTION_DOUBLE:
-      provide_short = " <REAL>";
-      provide_long = "=<REAL>";
+      provide = "<REAL>";
       break;
     case SC_OPTION_STRING:
-      provide_short = " <STRING>";
-      provide_long = "=<STRING>";
+      provide = "<STRING>";
       break;
     case SC_OPTION_INIFILE:
-      provide_short = " <INIFILE>";
-      provide_long = "=<INIFILE>";
+      provide = "<FILE>";
       break;
     case SC_OPTION_CALLBACK:
       if (item->has_arg) {
-        provide_short = " <ARG>";
-        provide_long = "=<ARG>";
+        provide = "<ARG>";
       }
       break;
+    case SC_OPTION_KEYVALUE:
+      provide = "<CHOICE>";
+      break;
     default:
       SC_ABORT_NOT_REACHED ();
     }
+#if 0
+    separator = item->has_arg ? "=" : "";
+#else
+    separator = "";
+#endif
     outbuf[0] = '\0';
     printed = 0;
     if (item->opt_char != '\0' && item->opt_name != NULL) {
-      printed = snprintf (outbuf, BUFSIZ, "   -%c%s | --%s%s",
-                          item->opt_char, provide_short,
-                          item->opt_name, provide_long);
+      printed +=
+        snprintf (outbuf + printed, BUFSIZ - printed, "   -%c | --%s%s",
+                  item->opt_char, item->opt_name, separator);
     }
     else if (item->opt_char != '\0') {
-      printed = snprintf (outbuf, BUFSIZ, "   -%c%s",
-                          item->opt_char, provide_short);
+      printed += snprintf (outbuf + printed, BUFSIZ - printed, "   -%c",
+                           item->opt_char);
     }
     else if (item->opt_name != NULL) {
-      printed = snprintf (outbuf, BUFSIZ, "   --%s%s",
-                          item->opt_name, provide_long);
+      printed += snprintf (outbuf + printed, BUFSIZ - printed, "   --%s%s",
+                           item->opt_name, separator);
     }
     else {
       SC_ABORT_NOT_REACHED ();
     }
+    printed += snprintf (outbuf + printed, BUFSIZ - printed, "%*s%s",
+                         SC_MAX (1, opt->space_type - printed), "", provide);
     if (item->help_string != NULL) {
-      snprintf (outbuf + printed, BUFSIZ - printed, "%*s%s",
-                SC_MAX (1, 40 - printed), "", item->help_string);
+      printed += snprintf (outbuf + printed, BUFSIZ - printed, "%*s%s",
+                           SC_MAX (1, opt->space_help - printed), "",
+                           item->help_string);
     }
     SC_GEN_LOGF (package_id, SC_LC_GLOBAL, log_priority, "%s\n", outbuf);
   }
@@ -555,15 +672,21 @@ sc_options_print_summary (int package_id, int log_priority,
 
   for (iz = 0; iz < count; ++iz) {
     item = (sc_option_item_t *) sc_array_index (items, iz);
-    if (item->opt_type == SC_OPTION_INIFILE) {
+    if (item->opt_type == SC_OPTION_INIFILE ||
+        item->opt_type == SC_OPTION_CALLBACK) {
       continue;
     }
+    printed = 0;
     if (item->opt_name == NULL) {
-      printed = snprintf (outbuf, BUFSIZ, "   -%c: ", item->opt_char);
+      printed += snprintf (outbuf + printed, BUFSIZ - printed, "   -%c",
+                           item->opt_char);
     }
     else {
-      printed = snprintf (outbuf, BUFSIZ, "   %s: ", item->opt_name);
+      printed += snprintf (outbuf + printed, BUFSIZ - printed, "   %s",
+                           item->opt_name);
     }
+    printed += snprintf (outbuf + printed, BUFSIZ - printed, "%*s",
+                         SC_MAX (1, opt->space_type - printed), "");
     switch (item->opt_type) {
     case SC_OPTION_SWITCH:
       bvalue = *(int *) item->opt_var;
@@ -598,6 +721,7 @@ sc_options_print_summary (int package_id, int log_priority,
       printed += snprintf (outbuf + printed, BUFSIZ - printed,
                            "%s", string_val);
       break;
+#if 0
     case SC_OPTION_CALLBACK:
       if (item->called) {
         string_val = item->has_arg ? item->string_value : "true";
@@ -608,6 +732,12 @@ sc_options_print_summary (int package_id, int log_priority,
       printed += snprintf (outbuf + printed, BUFSIZ - printed,
                            "%s", string_val);
       break;
+#endif
+    case SC_OPTION_KEYVALUE:
+      SC_ASSERT (item->string_value != NULL);
+      printed += snprintf (outbuf + printed, BUFSIZ - printed,
+                           "%s", item->string_value);
+      break;
     default:
       SC_ABORT_NOT_REACHED ();
     }
@@ -650,7 +780,6 @@ sc_options_load (int package_id, int err_priority,
   size_t             *zvalue;
   const char         *s, *key;
   char                skey[BUFSIZ], lkey[BUFSIZ];
-  sc_options_callback_t fn;
 
   dict = iniparser_load (inifile);
   if (dict == NULL) {
@@ -661,7 +790,8 @@ sc_options_load (int package_id, int err_priority,
 
   for (iz = 0; iz < count; ++iz) {
     item = (sc_option_item_t *) sc_array_index (items, iz);
-    if (item->opt_type == SC_OPTION_INIFILE) {
+    if (item->opt_type == SC_OPTION_INIFILE ||
+        item->opt_type == SC_OPTION_CALLBACK) {
       continue;
     }
 
@@ -761,6 +891,7 @@ sc_options_load (int package_id, int err_priority,
         *(const char **) item->opt_var = item->string_value = SC_STRDUP (s);
       }
       break;
+#if 0
     case SC_OPTION_CALLBACK:
       if (item->has_arg) {
         s = iniparser_getstring (dict, key, NULL);
@@ -782,6 +913,27 @@ sc_options_load (int package_id, int err_priority,
         return -1;
       }
       break;
+#endif
+    case SC_OPTION_KEYVALUE:
+      SC_ASSERT (item->string_value != NULL);
+      s = iniparser_getstring (dict, key, NULL);
+      if (s != NULL) {
+        /* lookup the key and see if the result is valid */
+        iserror = *(ivalue = (int *) item->opt_var);
+        *ivalue = sc_keyvalue_get_int_check ((sc_keyvalue_t *)
+                                             item->user_data, s, &iserror);
+        if (iserror) {
+          /* key not found or of the wrong type; this cannot be ignored */
+          SC_GEN_LOGF (package_id, SC_LC_GLOBAL, err_priority,
+                       "Invalid key %s for option %s in file: %s\n",
+                       s, key, inifile);
+          iniparser_freedict (dict);
+          return -1;
+        }
+        SC_FREE (item->string_value);
+        item->string_value = SC_STRDUP (s);
+      }
+      break;
     default:
       SC_ABORT_NOT_REACHED ();
     }
@@ -837,10 +989,8 @@ sc_options_save (int package_id, int err_priority,
     if (item->opt_type == SC_OPTION_STRING && item->string_value == NULL) {
       continue;
     }
-    if (item->opt_type == SC_OPTION_INIFILE) {
-      continue;
-    }
-    if (item->opt_type == SC_OPTION_CALLBACK && !item->called) {
+    if (item->opt_type == SC_OPTION_INIFILE ||
+        item->opt_type == SC_OPTION_CALLBACK) {
       continue;
     }
 
@@ -917,6 +1067,7 @@ sc_options_save (int package_id, int err_priority,
     case SC_OPTION_STRING:
       retval = fprintf (file, "%s\n", item->string_value);
       break;
+#if 0
     case SC_OPTION_CALLBACK:
       if (item->has_arg) {
         SC_ASSERT (item->string_value != NULL);
@@ -926,6 +1077,11 @@ sc_options_save (int package_id, int err_priority,
         retval = fprintf (file, "%s\n", "true");
       }
       break;
+#endif
+    case SC_OPTION_KEYVALUE:
+      SC_ASSERT (item->string_value != NULL);
+      retval = fprintf (file, "%s\n", item->string_value);
+      break;
     default:
       SC_ABORT_NOT_REACHED ();
     }
@@ -970,10 +1126,11 @@ int
 sc_options_parse (int package_id, int err_priority, sc_options_t * opt,
                   int argc, char **argv)
 {
-  int                 retval;
+  int                 retval, iserror;
   int                 position, printed;
   int                 c, option_index;
   int                 item_index = -1;
+  int                *ivalue;
   size_t              iz;
   long                ilong;
   long long           ilonglong;
@@ -1019,13 +1176,14 @@ sc_options_parse (int package_id, int err_priority, sc_options_t * opt,
       break;
     }
     if (c == '?') {             /* invalid option */
-      if (optopt == 0) {
+      if (optopt == '-' || !isprint (optopt)) {
         SC_GEN_LOG (package_id, SC_LC_GLOBAL, err_priority,
-                    "Encountered invalid long option\n");
+                    "Invalid long option or missing argument\n");
       }
       else {
         SC_GEN_LOGF (package_id, SC_LC_GLOBAL, err_priority,
-                     "Encountered invalid short option: -%c\n", optopt);
+                     "Invalid short option: -%c or missing argument\n",
+                     optopt);
       }
       retval = -1;
       break;
@@ -1136,6 +1294,21 @@ sc_options_parse (int package_id, int err_priority, sc_options_t * opt,
         retval = -1;            /* this ends option processing */
       }
       break;
+    case SC_OPTION_KEYVALUE:
+      SC_ASSERT (item->string_value != NULL);
+      iserror = *(ivalue = (int *) item->opt_var);
+      *ivalue = sc_keyvalue_get_int_check ((sc_keyvalue_t *) item->user_data,
+                                           optarg, &iserror);
+      if (iserror) {
+        SC_GEN_LOGF (package_id, SC_LC_GLOBAL, err_priority,
+                     "Error looking up: %s\n", optarg);
+        retval = -1;            /* this ends option processing */
+      }
+      else {
+        SC_FREE (item->string_value);
+        item->string_value = SC_STRDUP (optarg);
+      }
+      break;
     default:
       SC_ABORT_NOT_REACHED ();
     }
