@@ -4,6 +4,7 @@
   connected adaptive quadtrees or octrees in parallel.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
   Written by Carsten Burstedde, Lucas C. Wilcox, and Tobin Isaac
 
   p4est is free software; you can redistribute it and/or modify
@@ -47,6 +48,29 @@
 #include <sc_containers.h>
 #define _p4est_const _sc_const
 
+/*--------------------------------------------------------------------*/
+/*------------------------ QUERY API CHANGES -------------------------*/
+/*---- definitions to allow user code to query the p4est library -----*/
+
+/** We do no longer dereference unneeded pointers in p4est_transfer_.
+ */
+#define P4EST_COMM_TRANSFER_NULL
+
+/** The \ref p4est_connectivity_new_disk function now accepts a bool arg.
+ * The same holds for \ref p4est_wrap_new_disk.
+ */
+#define P4EST_CONN_DISK_PERIODIC
+
+/** The \ref p4est_search_local function replaces \ref p4est_search.
+ * The latter function is still available with updated internal semantics.
+ * Furthermore, we have added \ref p4est_search_partition to search
+ * the parallel partition without accessing any local elements,
+ * and \ref p4est_search_all that combines the two.
+ */
+#define P4EST_SEARCH_LOCAL
+
+/*--------------------------------------------------------------------*/
+
 SC_EXTERN_C_BEGIN;
 
 /** Typedef for quadrant coordinates. */
@@ -58,6 +82,7 @@ typedef int32_t     p4est_qcoord_t;
 #define P4EST_QCOORD_MIN INT32_MIN
 #define P4EST_QCOORD_MAX INT32_MAX
 #define P4EST_QCOORD_1   ((p4est_qcoord_t) 1)
+#define P4EST_QCOORD_ABS(x) ((p4est_qcoord_t) labs ((long) (x)))
 
 /** Typedef for counting topological entities (trees, tree vertices). */
 typedef int32_t     p4est_topidx_t;
@@ -69,6 +94,7 @@ typedef int32_t     p4est_topidx_t;
 #define P4EST_TOPIDX_MAX INT32_MAX
 #define P4EST_TOPIDX_FITS_32 1
 #define P4EST_TOPIDX_1   ((p4est_topidx_t) 1)
+#define P4EST_TOPIDX_ABS(x) ((p4est_topidx_t) labs ((long) (x)))
 
 /** Typedef for processor-local indexing of quadrants and nodes. */
 typedef int32_t     p4est_locidx_t;
@@ -79,6 +105,7 @@ typedef int32_t     p4est_locidx_t;
 #define P4EST_LOCIDX_MIN INT32_MIN
 #define P4EST_LOCIDX_MAX INT32_MAX
 #define P4EST_LOCIDX_1   ((p4est_locidx_t) 1)
+#define P4EST_LOCIDX_ABS(x) ((p4est_locidx_t) labs ((long) (x)))
 
 /** Typedef for globally unique indexing of quadrants. */
 typedef int64_t     p4est_gloidx_t;
@@ -89,6 +116,39 @@ typedef int64_t     p4est_gloidx_t;
 #define P4EST_GLOIDX_MIN INT64_MIN
 #define P4EST_GLOIDX_MAX INT64_MAX
 #define P4EST_GLOIDX_1   ((p4est_gloidx_t) 1)
+#define P4EST_GLOIDX_ABS(x) ((p4est_gloidx_t) llabs ((long long) (x)))
+
+/** Tags for MPI messages */
+typedef enum p4est_comm_tag
+{
+  P4EST_COMM_TAG_FIRST = SC_TAG_FIRST,
+  P4EST_COMM_COUNT_PERTREE = SC_TAG_LAST,
+  P4EST_COMM_BALANCE_FIRST_COUNT,
+  P4EST_COMM_BALANCE_FIRST_LOAD,
+  P4EST_COMM_BALANCE_SECOND_COUNT,
+  P4EST_COMM_BALANCE_SECOND_LOAD,
+  P4EST_COMM_PARTITION_GIVEN,
+  P4EST_COMM_PARTITION_WEIGHTED_LOW,
+  P4EST_COMM_PARTITION_WEIGHTED_HIGH,
+  P4EST_COMM_PARTITION_CORRECTION,
+  P4EST_COMM_GHOST_COUNT,
+  P4EST_COMM_GHOST_LOAD,
+  P4EST_COMM_GHOST_EXCHANGE,
+  P4EST_COMM_GHOST_EXPAND_COUNT,
+  P4EST_COMM_GHOST_EXPAND_LOAD,
+  P4EST_COMM_GHOST_SUPPORT_COUNT,
+  P4EST_COMM_GHOST_SUPPORT_LOAD,
+  P4EST_COMM_GHOST_CHECKSUM,
+  P4EST_COMM_NODES_QUERY,
+  P4EST_COMM_NODES_REPLY,
+  P4EST_COMM_SAVE,
+  P4EST_COMM_LNODES_TEST,
+  P4EST_COMM_LNODES_PASS,
+  P4EST_COMM_LNODES_OWNED,
+  P4EST_COMM_LNODES_ALL,
+  P4EST_COMM_TAG_LAST
+}
+p4est_comm_tag_t;
 
 /* some error checking possibly specific to p4est */
 #ifdef P4EST_ENABLE_DEBUG
@@ -101,12 +161,29 @@ typedef int64_t     p4est_gloidx_t;
   do { int _p4est_i = (int) (expression);                               \
        SC_CHECK_ABORT (_p4est_i, "Expected true: '" #expression "'");   \
   } while (0)
+#define P4EST_EXECUTE_ASSERT_INT(expression,ival)                       \
+  do { int _p4est_i = (int) (expression);                               \
+       SC_CHECK_ABORT ((ival) == _p4est_i,                              \
+                       "Expected '" #ival "': '" #expression "'");      \
+  } while (0)
+#define P4EST_EXECUTE_ASSERT_TOPIDX(expression,tval)                    \
+  do { p4est_topidx_t _p4est_t = (p4est_topidx_t) (expression);         \
+       SC_CHECK_ABORT ((tval) == _p4est_t,                              \
+                       "Expected '" #tval "': '" #expression "'");      \
+  } while (0)
+#define P4EST_DEBUG_EXECUTE(expression)                 \
+  do { (void) (expression); } while (0)
 #else
 #define P4EST_ASSERT(c) SC_NOOP ()
-#define P4EST_EXECUTE_ASSERT_FALSE(expression) \
+#define P4EST_EXECUTE_ASSERT_FALSE(expression)          \
   do { (void) (expression); } while (0)
-#define P4EST_EXECUTE_ASSERT_TRUE(expression) \
+#define P4EST_EXECUTE_ASSERT_TRUE(expression)           \
   do { (void) (expression); } while (0)
+#define P4EST_EXECUTE_ASSERT_INT(expression,ival)       \
+  do { (void) (expression); } while (0)
+#define P4EST_EXECUTE_ASSERT_TOPIDX(expression,tval)    \
+  do { (void) (expression); } while (0)
+#define P4EST_DEBUG_EXECUTE(expression) SC_NOOP ()
 #endif
 
 /* macros for memory allocation, will abort if out of memory */
