@@ -4,6 +4,7 @@
   connected adaptive quadtrees or octrees in parallel.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
   Written by Carsten Burstedde, Lucas C. Wilcox, and Tobin Isaac
 
   p4est is free software; you can redistribute it and/or modify
@@ -760,13 +761,137 @@ p4est_quadrant_is_inside_tree (p4est_tree_t * tree,
     return 0;
 
   /* check if the end of q is not after the last tree quadrant */
+  /* tree->last_desc is an upper right corner quadrant by construction.
+   * It is ok to compare with q. */
+#if 0
   p4est_quadrant_last_descendant (q, &desc, P4EST_QMAXLEVEL);
+#endif
   if (p4est_quadrant_compare (&tree->last_desc, q) < 0)
     return 0;
 
   return 1;
 }
 
+int
+p4est_quadrant_is_first_last (const p4est_quadrant_t * f,
+                              const p4est_quadrant_t * l,
+                              const p4est_quadrant_t * a)
+{
+  p4est_qcoord_t      w;
+
+  P4EST_ASSERT (p4est_quadrant_compare (f, l) <= 0);
+  P4EST_ASSERT (p4est_quadrant_is_equal (a, f) ||
+                p4est_quadrant_is_ancestor (a, f));
+  P4EST_ASSERT (p4est_quadrant_is_equal (a, l) ||
+                p4est_quadrant_is_ancestor (a, l));
+
+  /* is the first corner a match? */
+  if (a->x != f->x || a->y != f->y
+#ifdef P4_TO_P8
+      || a->z != f->z
+#endif
+    ) {
+    return 0;
+  }
+
+  /* is the last corner a match? */
+  w = P4EST_QUADRANT_LEN (a->level) - P4EST_QUADRANT_LEN (l->level);
+  P4EST_ASSERT (w >= 0);
+  if (a->x + w != l->x || a->y + w != l->y
+#ifdef P4_TO_P8
+      || a->z + w != l->z
+#endif
+    ) {
+    return 0;
+  }
+
+  /* yes, they both are matches */
+  return 1;
+}
+
+void
+p4est_quadrant_enlarge_first (const p4est_quadrant_t * a,
+                              p4est_quadrant_t * q)
+{
+  p4est_qcoord_t      w;
+#ifdef P4EST_ENABLE_DEBUG
+  p4est_quadrant_t    inp = *q;
+#endif
+
+  P4EST_ASSERT (p4est_quadrant_is_equal (a, q) ||
+                p4est_quadrant_is_ancestor (a, q));
+
+  w = q->x | q->y
+#ifdef P4_TO_P8
+    | q->z
+#endif
+    ;
+  while (a->level < q->level && (w & P4EST_QUADRANT_LEN (q->level)) == 0) {
+    --q->level;
+  }
+  P4EST_ASSERT (p4est_quadrant_is_extended (q));
+
+  P4EST_ASSERT (p4est_quadrant_is_equal (a, q) ||
+                p4est_quadrant_is_ancestor (a, q));
+  P4EST_ASSERT (p4est_quadrant_is_equal (q, &inp) ||
+                p4est_quadrant_is_ancestor (q, &inp));
+
+  /* verify that the first corner stayed the same */
+  P4EST_ASSERT (q->x == inp.x && q->y == inp.y
+#ifdef P4_TO_P8
+                && q->z == inp.z
+#endif
+    );
+}
+
+void
+p4est_quadrant_enlarge_last (const p4est_quadrant_t * a, p4est_quadrant_t * q)
+{
+  int                 inlevel;
+  p4est_qcoord_t      w;
+#ifdef P4EST_ENABLE_DEBUG
+  p4est_quadrant_t    inp = *q;
+#endif
+
+  P4EST_ASSERT (p4est_quadrant_is_equal (a, q) ||
+                p4est_quadrant_is_ancestor (a, q));
+
+  inlevel = (int) q->level;
+  w = q->x & q->y
+#ifdef P4_TO_P8
+    & q->z
+#endif
+    ;
+  while (a->level < q->level && (w & P4EST_QUADRANT_LEN (q->level)) != 0) {
+    --q->level;
+  }
+
+  /* move first corner to the lower left according to q's enlargement */
+  w = ~(P4EST_QUADRANT_LEN (q->level) - P4EST_QUADRANT_LEN (inlevel));
+  q->x &= w;
+  q->y &= w;
+#ifdef P4_TO_P8
+  q->z &= w;
+#endif
+  P4EST_ASSERT (p4est_quadrant_is_extended (q));
+
+  P4EST_ASSERT (p4est_quadrant_is_equal (a, q) ||
+                p4est_quadrant_is_ancestor (a, q));
+  P4EST_ASSERT (p4est_quadrant_is_equal (q, &inp) ||
+                p4est_quadrant_is_ancestor (q, &inp));
+
+  /* verify that the last corner stayed the same */
+#ifdef P4EST_ENABLE_DEBUG
+  P4EST_ASSERT (inlevel == inp.level);
+  P4EST_ASSERT (~w >= 0);
+  P4EST_ASSERT (q->x + ~w == inp.x && q->y + ~w == inp.y
+#ifdef P4_TO_P8
+                && q->z + ~w == inp.z
+#endif
+    );
+#endif
+}
+
 void
 p4est_quadrant_ancestor (const p4est_quadrant_t * q,
                          int level, p4est_quadrant_t * r)
@@ -822,6 +947,25 @@ p4est_quadrant_sibling (const p4est_quadrant_t * q, p4est_quadrant_t * r,
   P4EST_ASSERT (p4est_quadrant_is_extended (r));
 }
 
+void
+p4est_quadrant_child (const p4est_quadrant_t * q, p4est_quadrant_t * r,
+                      int child_id)
+{
+  const p4est_qcoord_t shift = P4EST_QUADRANT_LEN (q->level + 1);
+
+  P4EST_ASSERT (p4est_quadrant_is_extended (q));
+  P4EST_ASSERT (q->level < P4EST_QMAXLEVEL);
+  P4EST_ASSERT (child_id >= 0 && child_id < P4EST_CHILDREN);
+
+  r->x = child_id & 0x01 ? (q->x | shift) : q->x;
+  r->y = child_id & 0x02 ? (q->y | shift) : q->y;
+#ifdef P4_TO_P8
+  r->z = child_id & 0x04 ? (q->z | shift) : q->z;
+#endif
+  r->level = q->level + 1;
+  P4EST_ASSERT (p4est_quadrant_is_parent (q, r));
+}
+
 void
 p4est_quadrant_face_neighbor (const p4est_quadrant_t * q,
                               int face, p4est_quadrant_t * r)
@@ -1707,7 +1851,7 @@ p4est_quadrant_linear_id (const p4est_quadrant_t * quadrant, int level)
 #endif
 
   P4EST_ASSERT (p4est_quadrant_is_extended (quadrant));
-  P4EST_ASSERT ((int) quadrant->level >= level && level >= 0);
+  P4EST_ASSERT (0 <= level && level <= P4EST_MAXLEVEL);
 
   /* this preserves the high bits from negative numbers */
   x = quadrant->x >> (P4EST_MAXLEVEL - level);
