@@ -4,6 +4,7 @@
   connected adaptive quadtrees or octrees in parallel.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
   Written by Carsten Burstedde, Lucas C. Wilcox, and Tobin Isaac
 
   p4est is free software; you can redistribute it and/or modify
@@ -46,7 +47,7 @@ SC_EXTERN_C_BEGIN;
 #define P4EST_FACES (2 * P4EST_DIM)
 /** The number of children of a quadrant
  *
- * also the nmber of corners */
+ * also the number of corners */
 #define P4EST_CHILDREN 4
 /** The number of children/corners touching one face */
 #define P4EST_HALF (P4EST_CHILDREN / 2)
@@ -126,12 +127,14 @@ const char         *p4est_connect_type_string (p4est_connect_type_t btype);
  * [0][0]..[0][2]..[num_vertices-1][0]..[num_vertices-1][2].
  *
  * The corners are only stored when they connect trees.
+ * In this case tree_to_corner indexes into \a ctt_offset.
  * Otherwise the tree_to_corner entry must be -1 and this corner is ignored.
  * If num_corners == 0, tree_to_corner and corner_to_* arrays are set to NULL.
  *
  * The arrays corner_to_* store a variable number of entries per corner.
  * For corner c these are at position [ctt_offset[c]]..[ctt_offset[c+1]-1].
  * Their number for corner c is ctt_offset[c+1] - ctt_offset[c].
+ * The entries encode all trees adjacent to corner c.
  * The size of the corner_to_* arrays is num_ctt = ctt_offset[num_corners].
  *
  * The *_to_attr arrays may have arbitrary contents defined by the user.
@@ -204,14 +207,26 @@ extern const int    p4est_corner_face_corners[4][4];
 /** Store the faces for each child and corner, can be -1. */
 extern const int    p4est_child_corner_faces[4][4];
 
+/** Transform a face corner across one of the adjacent faces into a neighbor tree.
+ * This version expects the neighbor face and orientation separately.
+ * \param [in] fc   A face corner number in 0..1.
+ * \param [in] f    A face that the face corner number \a fc is relative to.
+ * \param [in] nf   A neighbor face that is on the other side of \f.
+ * \param [in] o    The orientation between tree boundary faces \a f and \nf.
+ * \return          The face corner number relative to the neighbor's face.
+ */
+int                 p4est_connectivity_face_neighbor_face_corner
+  (int fc, int f, int nf, int o);
+
 /** Transform a corner across one of the adjacent faces into a neighbor tree.
  * This version expects the neighbor face and orientation separately.
  * \param [in] c    A corner number in 0..3.
  * \param [in] f    A face number that touches the corner \a c.
  * \param [in] nf   A neighbor face that is on the other side of \f.
  * \param [in] o    The orientation between tree boundary faces \a f and \nf.
+ * \return          The number of the corner seen from the neighbor tree.
  */
-int                 p4est_connectivity_face_neighbor_corner_orientation
+int                 p4est_connectivity_face_neighbor_corner
   (int c, int f, int nf, int o);
 
 /** Allocate a connectivity structure.
@@ -233,6 +248,8 @@ p4est_connectivity_t *p4est_connectivity_new (p4est_topidx_t num_vertices,
  * \param [in] num_trees      Number of trees in the forest.
  * \param [in] num_corners    Number of tree-connecting corners.
  * \param [in] coff           Corner-to-tree offsets (num_corners + 1 values).
+ *                            This must always be non-NULL; in trivial cases
+ *                            it is just a pointer to a p4est_topix value of 0.
  * \return                    The connectivity is checked for validity.
  */
 p4est_connectivity_t *p4est_connectivity_new_copy (p4est_topidx_t
@@ -249,6 +266,21 @@ p4est_connectivity_t *p4est_connectivity_new_copy (p4est_topidx_t
                                                    const p4est_topidx_t * ctt,
                                                    const int8_t * ctc);
 
+/** Broadcast a connectivity structure that exists only on one process to all.
+ *  On the other processors, it will be allocated using p4est_connectivity_new.
+ *  \param [in] conn_in For the root process the connectivity to be broadcast,
+ *                      for the other processes it must be NULL.
+ *  \param [in] root    The rank of the process that provides the connectivity.
+ *  \param [in] comm    The MPI communicator.
+ *  \return             For the root process this is a pointer to \a conn_in.
+ *                      Else, a pointer to a newly allocated connectivity
+ *                      structure with the same values as \a conn_in on the
+ *                      root process.
+ */
+p4est_connectivity_t *p4est_connectivity_bcast (p4est_connectivity_t *
+                                                conn_in, int root,
+                                                sc_MPI_Comm comm);
+
 /** Destroy a connectivity structure.  Also destroy all attributes.
  */
 void                p4est_connectivity_destroy (p4est_connectivity_t *
@@ -335,6 +367,16 @@ p4est_connectivity_t *p4est_connectivity_new_periodic (void);
  */
 p4est_connectivity_t *p4est_connectivity_new_rotwrap (void);
 
+/** Create a connectivity structure for two trees being rotated
+ * w.r.t. each other in a user-defined way
+ * \param[in] l_face      index of left face
+ * \param[in] r_face      index of right face
+ * \param[in] orientation orientation of trees w.r.t. each other
+ */
+p4est_connectivity_t *p4est_connectivity_new_twotrees (int l_face,
+                                                       int r_face,
+                                                       int orientation);
+
 /** Create a connectivity structure for a three-tree mesh around a corner.
  */
 p4est_connectivity_t *p4est_connectivity_new_corner (void);
@@ -360,14 +402,31 @@ p4est_connectivity_t *p4est_connectivity_new_star (void);
 p4est_connectivity_t *p4est_connectivity_new_cubed (void);
 
 /** Create a connectivity structure for a five-tree flat spherical disk.
+ * This disk can just as well be used as a square to test non-Cartesian maps.
+ * Without any mapping this connectivity covers the square [-3, 3]**2.
+ * \note The API of this function has changed to accept two arguments.
+ *       You can query the #define P4EST_CONN_DISK_PERIODIC to check
+ *       whether the new version with the argument is in effect.
+ *
  * The ordering of the trees is as follows:   4
  *                                          1 2 3
  *                                            0.
+ *
+ * The outside x faces may be identified topologically.
+ * The outside y faces may be identified topologically.
+ * Both identifications may be specified simultaneously.
+ * The general shape and periodicity are the same as those obtained with
+ * \ref p4est_connectivity_new_brick (1, 1, periodic_a, periodic_b).
+ *
+ * \param [in] periodic_a       Bool to make disk periodic in x direction.
+ * \param [in] periodic_b       Bool to make disk periodic in y direction.
  */
-p4est_connectivity_t *p4est_connectivity_new_disk (void);
+p4est_connectivity_t *p4est_connectivity_new_disk (int periodic_a,
+                                                   int periodic_b);
 
-/** An m by n array with periodicity in x and y if periodic_a and periodic_b
- * are true, respectively.
+/** A rectangular m by n array of trees with configurable periodicity.
+ * The brick is periodic in x and y if periodic_a and periodic_b are true,
+ * respectively.
  */
 p4est_connectivity_t *p4est_connectivity_new_brick (int mi, int ni,
                                                     int periodic_a,
@@ -389,6 +448,18 @@ p4est_connectivity_t *p4est_connectivity_new_brick (int mi, int ni,
  */
 p4est_connectivity_t *p4est_connectivity_new_byname (const char *name);
 
+/** Uniformly refine a connectivity.
+ * This is useful if you would like to uniformly refine by something other
+ * than a power of 2.
+ *
+ * \param [in] conn         a valid connectivity
+ * \param [in] num_per_edge the number of new trees in each direction
+ *
+ * \return a refined connectivity.
+ */
+p4est_connectivity_t *p4est_connectivity_refine (p4est_connectivity_t * conn,
+                                                 int num_per_edge);
+
 /** Fill an array with the axis combination of a face neighbor transform.
  * \param [in]  iface       The number of the originating face.
  * \param [in]  nface       Encoded as nface = r * 4 + nf, where nf = 0..3 is
@@ -496,7 +567,7 @@ void                p4est_connectivity_permute (p4est_connectivity_t * conn,
  * \param [in]     ctype      determines when an edge exists in the dual graph
  *                            of the connectivity structure.
  */
-void                p4est_connectivity_reorder (MPI_Comm comm, int k,
+void                p4est_connectivity_reorder (sc_MPI_Comm comm, int k,
                                                 p4est_connectivity_t * conn,
                                                 p4est_connect_type_t ctype);
 
@@ -618,7 +689,7 @@ p4est_corner_array_index (sc_array_t * array, size_t it)
  * \endcode
  *
  * This code can be called two ways.  The first, when \c vertex==NULL and \c
- * tree_to_vertex==NULL, is used to count the number of tress and vertices in
+ * tree_to_vertex==NULL, is used to count the number of trees and vertices in
  * the connectivity to be generated by the \c .inp mesh in the \a stream.  The
  * second, when \c vertices!=NULL and \c tree_to_vertex!=NULL, fill \c vertices
  * and \c tree_to_vertex.  In this case \c num_vertices and \c num_trees need
