@@ -4,6 +4,7 @@
   connected adaptive quadtrees or octrees in parallel.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
   Written by Carsten Burstedde, Lucas C. Wilcox, and Tobin Isaac
 
   p4est is free software; you can redistribute it and/or modify
@@ -193,6 +194,9 @@ p4est_nodes_new_local (p4est_t * p4est)
               /* can't check for quadrants larger than the root */
               if (q->level == 0 && rlev < 0)
                 continue;
+              /* can't check for quadrants deeper than maxlevel */
+              if (q->level == P4EST_QMAXLEVEL && rlev > 0)
+                continue;
               /* can't check for quadrants larger unless child id
                * and corner line up
                */
@@ -273,7 +277,7 @@ p4est_node_canonicalize (p4est_t * p4est, p4est_topidx_t treeid,
   int                 contacts, face, corner;
   int                 ftransform[P4EST_FTRANSFORM];
   size_t              ctreez;
-  p4est_topidx_t      ntreeid, ntreeid2, lowest;
+  p4est_topidx_t      ntreeid, lowest;
   p4est_quadrant_t    tmpq, o;
 #ifdef P4_TO_P8
   int                 edge;
@@ -335,8 +339,8 @@ p4est_node_canonicalize (p4est_t * p4est, p4est_topidx_t treeid,
       continue;
     }
     /* Transform the node into the other tree's coordinates */
-    ntreeid2 = p4est_find_face_transform (conn, treeid, face, ftransform);
-    P4EST_ASSERT (ntreeid2 == ntreeid);
+    P4EST_EXECUTE_ASSERT_TOPIDX
+      (p4est_find_face_transform (conn, treeid, face, ftransform), ntreeid);
     p4est_quadrant_transform_face (n, &o, ftransform);
     if (ntreeid < lowest) {
       /* we have found a new owning tree */
