@@ -4,6 +4,7 @@
   connected adaptive quadtrees or octrees in parallel.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
   Written by Carsten Burstedde, Lucas C. Wilcox, and Tobin Isaac
 
   p4est is free software; you can redistribute it and/or modify
@@ -57,12 +58,12 @@ SC_EXTERN_C_BEGIN;
  */
 #define P8EST_LAST_OFFSET(l) (P8EST_ROOT_LEN - P8EST_QUADRANT_LEN (l))
 
-/** The 3D quadrant (i.e. octant) datatype */
+/** The 3D quadrant (i.e., octant) datatype */
 typedef struct p8est_quadrant
 {
-  /*@{*/
+  /*@{ */
   p4est_qcoord_t      x, y, z;  /**< coordinates */
-  /*@}*/
+  /*@} */
   int8_t              level,    /**< level of refinement */
                       pad8;     /**< padding */
   int16_t             pad16;    /**< padding */
@@ -127,11 +128,13 @@ typedef struct p8est
   sc_MPI_Comm         mpicomm;          /**< MPI communicator */
   int                 mpisize,          /**< number of MPI processes */
                       mpirank;          /**< this process's MPI rank */
+  int                 mpicomm_owned;    /**< flag if communicator is owned */
   size_t              data_size;        /**< size of per-quadrant p.user_data
                      (see p8est_quadrant_t::p8est_quadrant_data::user_data) */
   void               *user_pointer;     /**< convenience pointer for users,
                                              never touched by p4est */
 
+  long                revision;         /**< Gets bumped on mesh change */
   p4est_topidx_t      first_local_tree; /**< 0-based index of first local
                                              tree, must be -1 for an empty
                                              processor */
@@ -152,22 +155,33 @@ typedef struct p8est
   sc_array_t         *trees;          /**< array of all trees */
 
   sc_mempool_t       *user_data_pool; /**< memory allocator for user data */
-                                      /*   WARNING: This is NULL if data size
-                                                    equals zero. */
+  /*   WARNING: This is NULL if data size
+     equals zero. */
   sc_mempool_t       *quadrant_pool;  /**< memory allocator for temporary
                                            quadrants */
   p8est_inspect_t    *inspect;        /**< algorithmic switches */
 }
 p8est_t;
 
-/** Calculate memory usage of a forest structure.
+/** Calculate local memory usage of a forest structure.
+ * Not collective.  The memory used on the current rank is returned.
  * The connectivity structure is not counted since it is not owned;
  * use p8est_connectivity_memory_usage (p8est->connectivity).
- * \param [in] p8est    Forest structure.
+ * \param [in] p8est    Valid forest structure.
  * \return              Memory used in bytes.
  */
 size_t              p8est_memory_used (p8est_t * p8est);
 
+/** Return the revision counter of the forest.
+ * Not collective, even though the revision value is the same on all ranks.
+ * A newly created forest starts with a revision counter of zero.
+ * Every refine, coarsen, partition, and balance that actually changes the mesh
+ * increases the counter by one.  Operations with no effect keep the old value.
+ * \param [in] p8est    The forest must be valid.
+ * \return              Non-negative number.
+ */
+long                p8est_revision (p8est_t * p8est);
+
 /** Callback function prototype to initialize the quadrant's user data.
  * \param [in] p8est         the forest
  * \param [in] which_tree    the tree containing \a quadrant
@@ -266,10 +280,13 @@ void                p8est_destroy (p8est_t * p8est);
  * Copying of quadrant user data is optional.
  * If old and new data sizes are 0, the user_data field is copied regardless.
  * The inspect member of the copy is set to NULL.
+ * The revision counter of the copy is set to zero.
  *
  * \param [in]  copy_data  If true, data are copied.
  *                         If false, data_size is set to 0.
- * \return  Returns a valid p8est that does not depend on the input.
+ * \return  Returns a valid p8est that does not depend on the input,
+ *                         except for borrowing the same connectivity.
+ *                         Its revision counter is 0.
  */
 p8est_t            *p8est_copy (p8est_t * input, int copy_data);
 
@@ -345,11 +362,17 @@ void                p8est_balance (p8est_t * p8est,
  * The forest will be partitioned between processors such that they
  * have an approximately equal number of quadrants (or sum of weights).
  *
+ * On one process, the function noops and does not call the weight callback.
+ * Otherwise, the weight callback is called once per quadrant in order.
+ *
  * \param [in,out] p8est      The forest that will be partitioned.
  * \param [in]     allow_for_coarsening Slightly modify partition such that
  *                            quadrant families are not split between ranks.
  * \param [in]     weight_fn  A weighting function or NULL
  *                            for uniform partitioning.
+ *                            When running with mpisize == 1, never called.
+ *                            Otherwise, called in order for all quadrants
+ *                            if not NULL.
  */
 void                p8est_partition (p8est_t * p8est,
                                      int allow_for_coarsening,
@@ -371,6 +394,9 @@ unsigned            p8est_checksum (p8est_t * p8est);
  * header.  This makes the file depend on mpisize.  For changing this see
  * p8est_save_ext() in p8est_extended.h.
  *
+ * The revision counter is not saved to the file, since that would make files
+ * different that come from different revisions but store the same mesh.
+ *
  * \param [in] filename    Name of the file to write.
  * \param [in] p8est       Valid forest structure.
  * \param [in] save_data   If true, the element data is saved.
@@ -394,6 +420,8 @@ void                p8est_save (const char *filename, p8est_t * p8est,
  * that it was stored with.  The defaults can be changed with p8est_load_ext()
  * in p8est_extended.h.
  *
+ * The revision counter of the loaded p4est is set to zero.
+ *
  * \param [in] filename         Name of the file to read.
  * \param [in] mpicomm          A valid MPI communicator.
  * \param [in] data_size        Size of data for each quadrant which can be
