@@ -4,6 +4,7 @@
   connected adaptive quadtrees or octrees in parallel.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
   Written by Carsten Burstedde, Lucas C. Wilcox, and Tobin Isaac
 
   p4est is free software; you can redistribute it and/or modify
@@ -62,6 +63,20 @@ const int           p8est_face_permutation_refs[6][6] =
  { 0, 2, 2, 0, 0, 1 },
  { 2, 0, 0, 2, 2, 0 }};
 
+const int           p8est_face_edge_permutations[8][4] =
+{{ 0, 1, 2, 3 },
+ { 0, 1, 3, 2 },
+ { 1, 0, 2, 3 },
+ { 1, 0, 3, 2 },
+ { 2, 3, 0, 1 },
+ { 2, 3, 1, 0 },
+ { 3, 2, 0, 1 },
+ { 3, 2, 1, 0 }};
+const int           p8est_face_edge_permutation_sets[3][4] =
+{{ 4, 1, 2, 7 },
+ { 0, 6, 5, 3 },
+ { 0, 5, 6, 3 }};
+
 const int           p8est_edge_faces[12][2] =
 {{ 2, 4 },
  { 3, 4 },
@@ -88,6 +103,19 @@ const int           p8est_edge_corners[12][2] =
  { 1, 5 },
  { 2, 6 },
  { 3, 7 }};
+const int           p8est_edge_edge_corners[12][8] =
+{{  0,  1, -1, -1, -1, -1, -1, -1},
+ { -1, -1,  0,  1, -1, -1, -1, -1},
+ { -1, -1, -1, -1,  0,  1, -1, -1},
+ { -1, -1, -1, -1, -1, -1,  0,  1},
+ {  0, -1,  1, -1, -1, -1, -1, -1},
+ { -1,  0, -1,  1, -1, -1, -1, -1},
+ { -1, -1, -1, -1,  0, -1,  1, -1},
+ { -1, -1, -1, -1, -1,  0, -1,  1},
+ {  0, -1, -1, -1,  1, -1, -1, -1},
+ { -1,  0, -1, -1, -1,  1, -1, -1},
+ { -1, -1,  0, -1, -1, -1,  1, -1},
+ { -1, -1, -1,  0, -1, -1, -1,  1}};
 const int           p8est_edge_face_corners[12][6][2] =
 {{{ -1, -1 }, { -1, -1 }, {  0,  1 }, { -1, -1 }, {  0,  1 }, { -1, -1 }},
  {{ -1, -1 }, { -1, -1 }, { -1, -1 }, {  0,  1 }, {  2,  3 }, { -1, -1 }},
@@ -101,6 +129,19 @@ const int           p8est_edge_face_corners[12][6][2] =
  {{ -1, -1 }, {  0,  2 }, {  1,  3 }, { -1, -1 }, { -1, -1 }, { -1, -1 }},
  {{  1,  3 }, { -1, -1 }, { -1, -1 }, {  0,  2 }, { -1, -1 }, { -1, -1 }},
  {{ -1, -1 }, {  1,  3 }, { -1, -1 }, {  1,  3 }, { -1, -1 }, { -1, -1 }}};
+const int           p8est_edge_face_edges[12][6] =
+{{ -1, -1,  0, -1,  0, -1 },
+ { -1, -1, -1,  0,  1, -1 },
+ { -1, -1,  1, -1, -1,  0 },
+ { -1, -1, -1,  1, -1,  1 },
+ {  0, -1, -1, -1,  2, -1 },
+ { -1,  0, -1, -1,  3, -1 },
+ {  1, -1, -1, -1, -1,  2 },
+ { -1,  1, -1, -1, -1,  3 },
+ {  2, -1,  2, -1, -1, -1 },
+ { -1,  2,  3, -1, -1, -1 },
+ {  3, -1, -1,  2, -1, -1 },
+ { -1,  3, -1,  3, -1, -1 }};
 
 const int           p8est_corner_faces[8][3] =
 {{ 0, 2, 4 },
@@ -129,7 +170,15 @@ const int           p8est_corner_face_corners[8][6] =
  { -1,  2,  3, -1, -1,  1 },
  {  3, -1, -1,  2, -1,  2 },
  { -1,  3, -1,  3, -1,  3 }};
-
+const int           p8est_corner_edge_corners[8][12] =
+{{  0, -1, -1, -1,  0, -1, -1, -1,  0, -1, -1, -1 },
+ {  1, -1, -1, -1, -1,  0, -1, -1, -1,  0, -1, -1 },
+ { -1,  0, -1, -1,  1, -1, -1, -1, -1, -1,  0, -1 },
+ { -1,  1, -1, -1, -1,  1, -1, -1, -1, -1, -1,  0 },
+ { -1, -1,  0, -1, -1, -1,  0, -1,  1, -1, -1, -1 },
+ { -1, -1,  1, -1, -1, -1, -1,  0, -1,  1, -1, -1 },
+ { -1, -1, -1,  0, -1, -1,  1, -1, -1, -1,  1, -1 },
+ { -1, -1, -1,  1, -1, -1, -1,  1, -1, -1, -1,  1 }};
 const int           p8est_child_edge_faces[8][12] =
 {{ -1,  4,  2, -1, -1,  4,  0, -1, -1,  2,  0, -1 },
  { -1,  4,  2, -1,  4, -1, -1,  1,  2, -1, -1,  1 },
@@ -211,67 +260,6 @@ p8est_connectivity_new_unitcube (void)
                                       NULL, &num_ctt, NULL, NULL);
 }
 
-p4est_connectivity_t *
-p8est_connectivity_new_periodic (void)
-{
-  const p4est_topidx_t num_vertices = 8;
-  const p4est_topidx_t num_trees = 1;
-  const p4est_topidx_t num_edges = 3;
-  const p4est_topidx_t num_corners = 1;
-  const double        vertices[8 * 3] = {
-    0, 0, 0,
-    1, 0, 0,
-    0, 1, 0,
-    1, 1, 0,
-    0, 0, 1,
-    1, 0, 1,
-    0, 1, 1,
-    1, 1, 1,
-  };
-  const p4est_topidx_t tree_to_vertex[1 * 8] = {
-    0, 1, 2, 3, 4, 5, 6, 7,
-  };
-  const p4est_topidx_t tree_to_tree[1 * 6] = {
-    0, 0, 0, 0, 0, 0,
-  };
-  const int8_t        tree_to_face[1 * 6] = {
-    1, 0, 3, 2, 5, 4,
-  };
-  const p4est_topidx_t tree_to_edge[1 * 12] = {
-    0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2,
-  };
-  const p4est_topidx_t ett_offset[3 + 1] = {
-    0, 4, 8, 12,
-  };
-  const p4est_topidx_t edge_to_tree[12] = {
-    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-  };
-  const int8_t        edge_to_edge[12] = {
-    0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11,
-  };
-  const p4est_topidx_t tree_to_corner[1 * 8] = {
-    0, 0, 0, 0, 0, 0, 0, 0,
-  };
-  const p4est_topidx_t ctt_offset[1 + 1] = {
-    0, 8,
-  };
-  const p4est_topidx_t corner_to_tree[8] = {
-    0, 0, 0, 0, 0, 0, 0, 0,
-  };
-  const int8_t        corner_to_corner[8] = {
-    0, 1, 2, 3, 4, 5, 6, 7,
-  };
-
-  return p4est_connectivity_new_copy (num_vertices, num_trees,
-                                      num_edges, num_corners,
-                                      vertices, tree_to_vertex,
-                                      tree_to_tree, tree_to_face,
-                                      tree_to_edge, ett_offset,
-                                      edge_to_tree, edge_to_edge,
-                                      tree_to_corner, ctt_offset,
-                                      corner_to_tree, corner_to_corner);
-}
-
 p4est_connectivity_t *
 p8est_connectivity_new_rotwrap (void)
 {
@@ -874,47 +862,62 @@ p8est_find_edge_transform_internal (p4est_connectivity_t * conn,
                                     p8est_edge_info_t * ei,
                                     const p4est_topidx_t * ett,
                                     const int8_t * ete,
-                                    p4est_topidx_t edge_trees,
-                                    p4est_topidx_t ntrees[2])
+                                    p4est_topidx_t edge_trees)
 {
-  int                 i;
+  int                 i, j;
   int                 redge, nedge, iflip, nflip;
-  int                 pref, pset, fc[2];
-  int                 faces[2], nfaces[2], orients[2];
-#ifdef P4EST_ENABLE_DEBUG
-  int                 founds[2];
-#endif
-  int                 nows[2];
+  int                 pref, pset, fc[2], nc[2];
+  int                 face, nface, orient, eorient;
   p4est_topidx_t      etree, ietree, ntree;
   p8est_edge_transform_t *et;
   sc_array_t         *ta = &ei->edge_transforms;
-  const int           noncorners[2] = { -1, -1 };
-  const int          *fcorners[2] = { noncorners, noncorners };
-  const int          *nfcorners;
-  int                 flipped = 0;
+  const int          *fcorners;
+  int                 distinct = 1;
+  int                 edges[3], edgeorients[3];
+  p4est_topidx_t      etrees[3];
 
   P4EST_ASSERT (0 <= itree && itree < conn->num_trees);
   P4EST_ASSERT (0 <= iedge && iedge < P8EST_EDGES);
   P4EST_ASSERT (ta->elem_size == sizeof (p8est_edge_transform_t));
 
+  etrees[0] = itree;
+  edges[0] = iedge;
+  edgeorients[0] = 0;
+
   /* identify touching faces */
   for (i = 0; i < 2; ++i) {
-    faces[i] = p8est_edge_faces[iedge][i];
-    ntrees[i] = conn->tree_to_tree[P4EST_FACES * itree + faces[i]];
-    nfaces[i] = (int) conn->tree_to_face[P4EST_FACES * itree + faces[i]];
-    if (ntrees[i] == itree && nfaces[i] == faces[i]) {  /* domain boundary */
-      ntrees[i] = -1;
-      nfaces[i] = orients[i] = -1;
+    face = p8est_edge_faces[iedge][i];
+    ntree = conn->tree_to_tree[P4EST_FACES * itree + face];
+    nface = (int) conn->tree_to_face[P4EST_FACES * itree + face];
+    if (ntree != itree || nface != face) {      /* not domain boundary */
+      orient = nface / P4EST_FACES;
+      nface %= P4EST_FACES;
+      fcorners = &(p8est_edge_face_corners[iedge][face][0]);
+      P4EST_ASSERT (fcorners[0] >= 0 && fcorners[1] >= 0);
+      pref = p8est_face_permutation_refs[face][nface];
+      pset = p8est_face_permutation_sets[pref][orient];
+      fc[0] = p8est_face_permutations[pset][fcorners[0]];
+      fc[1] = p8est_face_permutations[pset][fcorners[1]];
+
+      /* if this is a new edge, add it */
+      nc[0] = p8est_face_corners[nface][fc[0]];
+      nc[1] = p8est_face_corners[nface][fc[1]];
+      nedge = p8est_child_corner_edges[nc[0]][nc[1]];
+      P4EST_ASSERT (nedge >= 0);
+      eorient = (p8est_edge_corners[nedge][1] == nc[0]);
+      for (j = 0; j < distinct; j++) {
+        if (ntree == etrees[j] &&
+            nedge == edges[j] && eorient == edgeorients[j]) {
+          break;
+        }
+      }
+      if (j == distinct) {
+        etrees[j] = ntree;
+        edges[j] = nedge;
+        edgeorients[j] = eorient;
+        distinct++;
+      }
     }
-    else {
-      orients[i] = nfaces[i] / P4EST_FACES;
-      nfaces[i] %= P4EST_FACES;
-      fcorners[i] = p8est_edge_face_corners[iedge][faces[i]];
-      P4EST_ASSERT (fcorners[i][0] >= 0 && fcorners[i][1] >= 0);
-    }
-#ifdef P4EST_ENABLE_DEBUG
-    founds[i] = 0;
-#endif
   }
 
   /* find orientation of this edge */
@@ -946,6 +949,16 @@ p8est_find_edge_transform_internal (p4est_connectivity_t * conn,
     nedge = redge % P8EST_EDGES;
     nflip = (redge / P8EST_EDGES) ^ iflip;
 
+    for (j = 0; j < distinct; j++) {
+      if (ntree == etrees[j] && nedge == edges[j] && nflip == edgeorients[j]) {
+        break;
+      }
+    }
+    if (j < distinct) {
+      /* already found from self or faces */
+      continue;
+    }
+#if 0
     nows[0] = nows[1] = 0;
     for (i = 0; i < 2; ++i) {
       if (ntree == ntrees[i]) {
@@ -974,6 +987,7 @@ p8est_find_edge_transform_internal (p4est_connectivity_t * conn,
     if (nows[0] || nows[1]) {
       continue;
     }
+#endif
 
     /* else we have a diagonal edge with ntree */
     et = (p8est_edge_transform_t *) sc_array_push (ta);
@@ -986,18 +1000,85 @@ p8est_find_edge_transform_internal (p4est_connectivity_t * conn,
     et->corners = (int8_t) (nedge % 4);
   }
 
-  return flipped;
+  return distinct;
 }
 
 #include "p4est_connectivity.c"
 
+int
+p8est_connectivity_face_neighbor_face_edge (int fe, int f, int nf, int o)
+{
+  int                 nfe, pref, pset;
+
+  P4EST_ASSERT (0 <= fe && fe < P4EST_HALF);
+  P4EST_ASSERT (0 <= f && f < P4EST_FACES);
+  P4EST_ASSERT (0 <= nf && nf < P4EST_FACES);
+  P4EST_ASSERT (0 <= o && o < P4EST_HALF);
+
+  pref = p8est_face_permutation_refs[f][nf];
+  pset = p8est_face_edge_permutation_sets[pref][o];
+  nfe = p8est_face_edge_permutations[pset][fe];
+
+  P4EST_ASSERT (0 <= nfe && nfe < P4EST_HALF);
+
+  return nfe;
+}
+
+int
+p8est_connectivity_face_neighbor_edge (int e, int f, int nf, int o)
+{
+  int                 fe, nfe;
+
+  P4EST_ASSERT (0 <= e && e < P8EST_EDGES);
+  P4EST_ASSERT (0 <= f && f < P4EST_FACES);
+  P4EST_ASSERT (0 <= nf && nf < P4EST_FACES);
+  P4EST_ASSERT (0 <= o && o < P4EST_HALF);
+
+  fe = p8est_edge_face_edges[e][f];
+  P4EST_ASSERT (0 <= fe && fe < P4EST_HALF);
+
+  nfe = p8est_connectivity_face_neighbor_face_edge (fe, f, nf, o);
+
+  return p8est_face_edges[nf][nfe];
+}
+
+int
+p8est_connectivity_edge_neighbor_edge_corner (int ec, int o)
+{
+  int                 nec;
+
+  P4EST_ASSERT (0 <= ec && ec < 2);
+  P4EST_ASSERT (0 <= o && o < 2);
+
+  nec = ec ^ o;
+  P4EST_ASSERT (0 <= nec && nec < 2);
+
+  return nec;
+}
+
+int
+p8est_connectivity_edge_neighbor_corner (int c, int e, int ne, int o)
+{
+  int                 ec, nec;
+
+  P4EST_ASSERT (0 <= c && c < P4EST_CHILDREN);
+  P4EST_ASSERT (0 <= e && e < P8EST_EDGES);
+
+  ec = p8est_corner_edge_corners[c][e];
+  P4EST_ASSERT (0 <= ec && ec < 2);
+
+  nec = p8est_connectivity_edge_neighbor_edge_corner (ec, o);
+  P4EST_ASSERT (0 <= nec && nec < 2);
+
+  return p8est_edge_corners[ne][nec];
+}
+
 void
 p8est_find_edge_transform (p4est_connectivity_t * conn,
                            p4est_topidx_t itree, int iedge,
                            p8est_edge_info_t * ei)
 {
-  int                 flipped;
-  p4est_topidx_t      ntrees[2], edge_trees, aedge, ettae;
+  p4est_topidx_t      edge_trees, aedge, ettae;
   sc_array_t         *ta = &ei->edge_transforms;
 
   P4EST_ASSERT (0 <= itree && itree < conn->num_trees);
@@ -1022,12 +1103,12 @@ p8est_find_edge_transform (p4est_connectivity_t * conn,
   P4EST_ASSERT (0 <= ettae && 1 <= edge_trees);
 
   /* loop through all edge neighbors and find edge connections */
-  flipped = p8est_find_edge_transform_internal (conn, itree, iedge, ei,
-                                                conn->edge_to_tree + ettae,
-                                                conn->edge_to_edge + ettae,
-                                                edge_trees, ntrees);
-  P4EST_ASSERT (edge_trees == (p4est_topidx_t) ta->elem_count
-                + 1 + (ntrees[0] != -1) + (ntrees[1] != -1) - flipped);
+  P4EST_EXECUTE_ASSERT_INT
+    (p8est_find_edge_transform_internal (conn, itree, iedge, ei,
+                                         conn->edge_to_tree + ettae,
+                                         conn->edge_to_edge + ettae,
+                                         edge_trees),
+     (int) edge_trees - (int) ta->elem_count);
 }
 
 int
