@@ -4,6 +4,7 @@
   connected adaptive quadtrees or octrees in parallel.
 
   Copyright (C) 2010 The University of Texas System
+  Additional copyright (C) 2011 individual authors
   Written by Carsten Burstedde, Lucas C. Wilcox, and Tobin Isaac
 
   p4est is free software; you can redistribute it and/or modify
@@ -88,8 +89,8 @@ p8est_ghost_t;
  * \param [in] ghost    Ghost layer structure.
  * \return true if \a ghost is valid
  */
-int                 p8est_ghost_is_valid (p8est_t * p8est, p8est_ghost_t *ghost);
-
+int                 p8est_ghost_is_valid (p8est_t * p8est,
+                                          p8est_ghost_t * ghost);
 
 /** Calculate the memory usage of the ghost layer.
  * \param [in] ghost    Ghost layer structure.
@@ -250,6 +251,41 @@ void                p8est_ghost_exchange_data (p8est_t * p4est,
                                                p8est_ghost_t * ghost,
                                                void *ghost_data);
 
+/** Transient storage for asynchronous ghost exchange. */
+typedef struct p8est_ghost_exchange
+{
+  int                 is_custom;        /**< False for p4est_ghost_exchange_data */
+  int                 is_levels;        /**< Are we restricted to levels or not */
+  p8est_t            *p4est;
+  p8est_ghost_t      *ghost;
+  int                 minlevel, maxlevel;       /**< Meaningful with is_levels */
+  size_t              data_size;
+  void               *ghost_data;
+  int                *qactive, *qbuffer;
+  sc_array_t          requests, sbuffers;
+  sc_array_t          rrequests, rbuffers;
+}
+p8est_ghost_exchange_t;
+
+/** Begin an asynchronous ghost data exchange by posting messages.
+ * The arguments are identical to p8est_ghost_exchange_data.
+ * The return type is always non-NULL and must be passed to
+ * p8est_ghost_exchange_data_end to complete the exchange.
+ * The ghost data must not be accessed before completion.
+ * \param [in,out]  ghost_data  Must stay alive into the completion call.
+ * \return          Transient storage for messages in progress.
+ */
+p8est_ghost_exchange_t *p8est_ghost_exchange_data_begin
+  (p8est_t * p4est, p8est_ghost_t * ghost, void *ghost_data);
+
+/** Complete an asynchronous ghost data exchange.
+ * This function waits for all pending MPI communications.
+ * \param [in,out]  Data created ONLY by p8est_ghost_exchange_data_begin.
+ *                  It is deallocated before this function returns.
+ */
+void                p8est_ghost_exchange_data_end
+  (p8est_ghost_exchange_t * exc);
+
 /** Transfer data for local quadrants that are ghosts to other processors.
  * The data size is the same for all quadrants and can be chosen arbitrarily.
  * \param [in] p8est            The forest used for reference.
@@ -266,6 +302,29 @@ void                p8est_ghost_exchange_custom (p8est_t * p4est,
                                                  void **mirror_data,
                                                  void *ghost_data);
 
+/** Begin an asynchronous ghost data exchange by posting messages.
+ * The arguments are identical to p8est_ghost_exchange_custom.
+ * The return type is always non-NULL and must be passed to
+ * p8est_ghost_exchange_custom_end to complete the exchange.
+ * The ghost data must not be accessed before completion.
+ * The mirror data can be safely discarded right after this function returns
+ * since it is copied into internal send buffers.
+ * \param [in]      mirror_data Not required to stay alive any longer.
+ * \param [in,out]  ghost_data  Must stay alive into the completion call.
+ * \return          Transient storage for messages in progress.
+ */
+p8est_ghost_exchange_t *p8est_ghost_exchange_custom_begin
+  (p8est_t * p4est, p8est_ghost_t * ghost,
+   size_t data_size, void **mirror_data, void *ghost_data);
+
+/** Complete an asynchronous ghost data exchange.
+ * This function waits for all pending MPI communications.
+ * \param [in,out]  Data created ONLY by p8est_ghost_exchange_custom_begin.
+ *                  It is deallocated before this function returns.
+ */
+void                p8est_ghost_exchange_custom_end
+  (p8est_ghost_exchange_t * exc);
+
 /** Transfer data for local quadrants that are ghosts to other processors.
  * The data size is the same for all quadrants and can be chosen arbitrarily.
  * This function restricts the transfer to a range of refinement levels.
@@ -275,7 +334,7 @@ void                p8est_ghost_exchange_custom (p8est_t * p4est,
  * \param [in] minlevel         Level of the largest quads to be exchanged.
  *                              Use <= 0 for no restriction.
  * \param [in] maxlevel         Level of the smallest quads to be exchanged.
- *                              Use >= P4EST_QMAXLEVEL for no restriction.
+ *                              Use >= P8EST_QMAXLEVEL for no restriction.
  * \param [in] data_size        The data size to transfer per quadrant.
  * \param [in] mirror_data      One data pointer per mirror quadrant as input. 
  * \param [in,out] ghost_data   Pre-allocated contiguous data for all ghosts
@@ -290,6 +349,29 @@ void                p8est_ghost_exchange_custom_levels (p8est_t * p8est,
                                                         void **mirror_data,
                                                         void *ghost_data);
 
+/** Begin an asynchronous ghost data exchange by posting messages.
+ * The arguments are identical to p8est_ghost_exchange_custom_levels.
+ * The return type is always non-NULL and must be passed to
+ * p8est_ghost_exchange_custom_levels_end to complete the exchange.
+ * The ghost data must not be accessed before completion.
+ * The mirror data can be safely discarded right after this function returns
+ * since it is copied into internal send buffers.
+ * \param [in]      mirror_data Not required to stay alive any longer.
+ * \param [in,out]  ghost_data  Must stay alive into the completion call.
+ * \return          Transient storage for messages in progress.
+ */
+p8est_ghost_exchange_t *p8est_ghost_exchange_custom_levels_begin
+  (p8est_t * p4est, p8est_ghost_t * ghost, int minlevel, int maxlevel,
+   size_t data_size, void **mirror_data, void *ghost_data);
+
+/** Complete an asynchronous ghost data exchange.
+ * This function waits for all pending MPI communications.
+ * \param [in,out]  Data created ONLY by p8est_ghost_exchange_custom_levels_begin.
+ *                  It is deallocated before this function returns.
+ */
+void                p8est_ghost_exchange_custom_levels_end
+  (p8est_ghost_exchange_t * exc);
+
 /** Expand the size of the ghost layer and mirrors by one additional layer of
  * adjacency.
  * \param [in] p8est            The forest from which the ghost layer was
